//
//   Version 1.0
//
//   Copyright 2016-2018 Lee Djavaherian
//
//   This program is distributed under the terms of the GNU General Public License
//   This file is part of TrillSat.
//
//   TrillSat is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   TrillSat is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with TrillSat.  If not, see <http://www.gnu.org/licenses/>.
///////////////////////////////////////////////////////////////////////////////////

//change to 200 to render most parts (or 100 to render Gyrofan)
$fn=20;

//set the showpart variable to one of the numbers below
//and then click preview or render buttons to see output
showpart = 100;

// 100   show full assembly
// 101   show full assembly with tether (to calibrate angles)
// 102   show exploded view (without Circuit Box covers)
// 200   show animation (after this is set, select Animate, then set FPS to 24, and set steps to 360)

// 1     gondola
// 3     end_cap
// 4     pulley_frame
// 6     batt_frame
// 8     radio_box
// 10    switching_box (the Power Regulation Box)
// 11    cpu_holder
// 13    corner_trim
// 14    side_trim_right_wire
// 15    side_trim_left_wire
// 16    brace
// 17    acrylic_separator
// 19    ldr_mount
// 20    gyro
// 21    capstan
// 22    planet
// 23    magnet_holder
// 24    qi_holder (the Qi Dock)

// 25    hex_nut_calibrator (initial calibration tool)
// 26    aluminum_bar_template (drill template tool)
// 27    side_trim_template (drill template tool)
// 28    corner_trim_template (drill template tool)

///////////////////////// INITIAL VARIABLES ///////////////////////////////////

//set variable for animation
if (showpart == 200) {$t=0;}

//Set values in millimeters
//Hard-coded values such as .9, 9.9 and 99.9 are numbers I
//added to tweak values 

screwdriver_shaft_diameter = 11;
magnet_diameter = 7;
magnet_holder_height = 11;
magnet_holder_thickness = 2.5;
magnet_holder_looseness = .3; //larger values looser

hex_diameter_screwdriver_shaft = 8;
hex_diameter_6_32 = 9;
hex_diameter_8_32 = 10;

bb_diameter = 4.4 + .5;

bearing_outer_diameter = 13;
bearing_width = 6;
bearing_inner_diameter = 4;

capstan_diameter = 38;
capstan_rod_diameter = 6.35;

//6-32 diameter
outer_diameter_6_32 = 3.5052;
inner_diameter_6_32 = 2.51206;

//8-32 diameter
outer_diameter_8_32 = 4.1656;
inner_diameter_8_32 = 3.16992;

screw_height = 4;
base_thickness = 2;

solar_panel_length = 555 + .2;
solar_panel_width = 136 + 3;
solar_panel_thickness = 5 + 1.7;

radio_thickness = 25 + 2;
radio_width = 48 + 3;
radio_length = 105 + 3;
radio_screw_mount_width = 42.5;
radio_screw_mount_height = 15 + 7; //allow room for audio interface underneath
radio_screw_mount_diameter = 6;
radio_screw_mount_inner_diameter = 3 * .67;

radio_wire_length = 37; //longer than needed
radio_wire_width = 25;

esp_width = 25 + 2;
esp_length = 38 + 2;
esp_mount_length = 33;
esp_mount_width = 21;
esp_mount_height = screw_height; 
esp_screw_mount_diameter = 5;
esp_screw_mount_inner_diameter = 2 * .67;


//note that esp height matches radio thickness
//so 25 mm is a good value for inside depth of circuit boxes
// but need to add extra depth for mount screws

//5 is minimum height for screws
//also add 7mm to allow extra space for audio interface under radio
esp_height = 25 + screw_height + 7;  //with jumpers attached

pi_length = 69 + 10; //add extra 10 cm for camera ribbon if used
pi_width = 31 + 2;

pi_mount_length = 58;
pi_mount_width = 22.5;
pi_mount_height = screw_height; 
pi_screw_mount_diameter = 6;
pi_screw_mount_inner_diameter = 2 * .67;

pi_wire_length = 35;
pi_wire_width = 25;

batt_board_width = 45 + 2;
batt_board_length = 57 + 2;
batt_board_screw_mount_length = 47;
batt_board_mount_length = 47.5;
batt_board_mount_width = 23.5;
batt_board_screw_mount_diameter = 4;
batt_board_screw_mount_inner_diameter = 2 * .9;
batt_board_post_diameter = .8;
batt_board_screw_mount_height = screw_height;
batt_board_wire_length = 50 + 2;
batt_board_wire_width = 45;

cpu_holder_thickness = 2;
cpu_holder_inset_width = 29;
cpu_holder_inset_length = 32;
cpu_holder_inset_depth = 5;

solar_board_screw_mount1_length = 43.5;
solar_board_screw_mount2_length = 20;
solar_board_screw_mount_width = 21.5;
solar_board_screw_mount2_offset = 12;
solar_board_screw_mount_diameter = 6.5;
solar_board_screw_mount_inner_diameter = 4 * .67;
solar_board_screw_mount_height = screw_height;


dc_boost_length = 60 + 2;
dc_boost_width = 38 + 2;
dc_boost_screw_mount_length = 45;
dc_boost_screw_mount_width = 30;
dc_boost_screw_mount_diameter = 7;
dc_boost_screw_mount_inner_diameter = 3 * .75;
dc_boost_mount_height = screw_height;

circuit_board_width = 50 + 2;
circuit_board_length = 60 + 2;
circuit_board_mount_width = 39;
circuit_board_mount_length = 50;
circuit_board_mount_height = screw_height;
circuit_board_screw_mount_diameter = 7;
circuit_board_screw_mount_inner_diameter = 3 * .67;

circuit_board2_width = 64 + 2;
circuit_board2_length = 82 + 2;
circuit_board2_mount_width = 53;
circuit_board2_mount_length = 72.5;
circuit_board2_mount_height = screw_height;
circuit_board2_screw_mount_diameter = 7;
circuit_board2_screw_mount_inner_diameter = 3 * .75;



pi_camera_length = 25 + 2;
pi_camera_width = 24 + 2;
pi_camera_height = 9;
pi_camera_mount_length = 20.8;
pi_camera_mount_width = 12.5;
pi_camera_mount_height = esp_height - 6;
pi_camera_screw_mount_diameter = 5;
pi_camera_screw_mount_inner_diameter = 2.2 * .67;
pi_camera_wire_width = 16 + 1;
pi_camera_wire_length = 70;



light_length = 20 + 2;
light_width = 20 + 2;
light_height = 13;

thin_trim_thickness = 3;
trim_thickness = thin_trim_thickness * 2 + solar_panel_thickness;  //you can see this on corner edges
echo ("trim thickness is:", trim_thickness);
thin_trim_height = 10;

 

rope_hole_diameter = 6;

solar_panel_mount_width = solar_panel_width - trim_thickness;


motor_mount_depth = 25 + .2;
motor_board_length = 70 + .2;
motor_board_width = 34 + .2;
motor_hub_diameter = 10 + .4;
motor_hub_depth = 4 + .4;



//motor_mount_depth + motor_hub_depth + 2; //the 2 is a spacer between the screwdriver motor
cylinder_thickness = 10;
fin_thickness = 5;
lop_off = 20;
max_print_width = 190;
max_print_height = 145;



//compute this to fit, uses leftover from side trim width (which uses max_print_width)
corner_trim_height = solar_panel_length/2 - max_print_width + thin_trim_thickness;
echo (corner_trim_height);

batt_box_thickness = 1.5;
batt_frame_thickness = 10;
circuit_box_thickness = 1.5;


endcap_drip_edge = 3;
endcap_height = 3;
endcap_shaft_hole = 11.5;
endcap_cone_small_diameter = 29;
endcap_cone_large_diameter = 34.5;
endcap_cone_depth = 10;

endcap_magnet_cylinder_diameter = 19;
endcap_hall_height = 3 + 7; //this also has to account for magnet height while spinning
endcap_hall_width = 4 + 1;
endcap_hall_thickness = 1 + .5;


circuit_box_drip_edge = 6; //set higher for drip edge
batt_box_drip_edge = 4;


//treat this as a rectangle
screwdriver_orange_width = 37 + .4;
screwdriver_orange_length = 47 + .4;  //also same as max diameter on black round part
screwdriver_cylinder_start = 24 + .2;  //point to add larger part of screwdriver cylinder
screwdriver_length = 109 + 1 + .2; //length from orange end to end of shaft ring (not including moving shaft), the +1 is for the motor board

accelerometer_length = 20 + .9;
accelerometer_width = 20 + .9;
accelerometer_screw_mount_diameter = 4;
accelerometer_screw_mount_inner_diameter = 2 * .8;
accelerometer_mount_width = 15.5;
accelerometer_mount_length = 15.5;
//this is based on where it things the rope hole is from 0, or bottom of solar panel
accelerometer_mount_height = rope_hole_diameter - 2;


batt_width = 56 + 2;
batt_length = 68 + 2;
batt_thickness = 19 + .2;
batt_air_thickness = 15;

batt_box_height = batt_thickness + base_thickness + batt_box_drip_edge;
batt_box_length = batt_width * 2 + batt_box_thickness * 4 + accelerometer_length;

circuit_box_width = solar_panel_mount_width + (trim_thickness - thin_trim_thickness) * 2;
echo ("circuit box width ", circuit_box_width);

//used fixed number to tweak angle to match gondola angle (11 seems to match)
batt_box_angle_width = (max_print_width - batt_box_length)/2 - 0;

//fits more tightly - fixed
max_gondola_width = solar_panel_mount_width - batt_box_thickness * 2 - .5;
max_gondola_length = batt_box_length - batt_box_thickness * 2 - .5;

brace_height = 15; //test
brace_width = 46;
brace_thickness = 15;

brace_flange_length = 15;
brace_flange_thickness = 3;

trim_overhang = brace_height + 5;

 vent_slot_length = 33;  
 vent_slot_height = brace_height - thin_trim_thickness;
 vent_slot_start = 25;
 vent_slot_gap = 20; 
 
 wire_flange_height = 14;
 wire_flange_length = vent_slot_length + 36;
 wire_bundle_height = (wire_flange_height + thin_trim_height - thin_trim_thickness * 2);
 wire_bundle_width = 5;

circuit_box_length = max_print_width - 25; // tweak the last number to shorten as necessary
echo ("circuit box length ", circuit_box_length);
circuit_box_height = base_thickness + esp_height + circuit_box_thickness + circuit_box_drip_edge; //it is as high as the thickest part, the radio

gyro_diameter = 120; //diameter of a compact disc - hole only
gyro_disc_diameter = 115; //allow 5 mm clearance (2.5 mm on the edges)
gyro_hole_diameter = 15;
gyro_mount_diameter = 29; //largest diameter of plastic mount on motor
gyro_thickness = 0;
gyro_center_ring_thickness = 3;
shot_diameter = 9.525 + .8;  //3/8 inch steel slingshot shot (make a little bigger of .4 around the edges so sphere holes allow snug but not too tight fit

acrylic_separator_base_width = 7;
acrylic_separator_tab_width = 10;
acrylic_separator_tab_length = 25;

temp_probe_diameter = 4.5 + 1.8; // i also use this diameter for solar power powers
cds_offset = 19;

screw_eye_diameter = 4.9; // for 3/16 eyebolt
screw_eye_inset_diameter = 12;
screw_eye_inset_depth = 6;

ldr_tube_length = 25;
ldr_tube_thickness = 2;
ldr_diameter = 5 + 2; //add 2 mm to allow space to work with
ldr_lens_diameter = 12;
ldr_tube_diameter = ldr_lens_diameter + ldr_tube_thickness * 2;

echo ("circuit box usable width is", circuit_box_width - circuit_box_thickness*2);
echo ("circuit box usable length is", circuit_box_length - circuit_box_thickness*2);

//////////////////////////// INDIVIDUAL PARTS //////////////////

if (showpart < 100) {
    if (showpart == 1) gondola();
    if (showpart == 3) end_cap();
    if (showpart == 4) pulley_frame();
    if (showpart == 6) batt_frame();
    if (showpart == 8) radio_box();
    if (showpart == 10) switching_box();
    if (showpart == 11) cpu_holder();
    if (showpart == 13) corner_trim();
    if (showpart == 14) side_trim_right_wire();
    if (showpart == 15) side_trim_left_wire();
    if (showpart == 16) brace();
    if (showpart == 17) acrylic_separator();
    if (showpart == 19) ldr_mount();
    if (showpart == 20) gyro();
    if (showpart == 21) capstan();
    if (showpart == 22) planet();
    if (showpart == 23) magnet_holder();
    if (showpart == 24) qi_holder();
    if (showpart == 25) hex_nut_calibrator();
    if (showpart == 26) aluminum_bar_template();
    if (showpart == 27) side_trim_template();
    if (showpart == 28) corner_trim_template();
} else {
    
    //animation settings
    if (showpart == 200) {
        
      rotate ([0, 22,0])                // starting angle
      rotate ([0, $t*45,0])             // pitch (circular)
      translate ([-$t*200,0,0])         // linear speed
      mirror ([0,0,1]) mirror ([1,0,0]) 
      rotate ([60*sin($t*360), 0, 0])   // roll (sinusoidal)

      assemble();
    }
    else rotate([0,0,180]) assemble();

}

//////////////////////////// ASSEMBLE MULTIPLE PARTS /////////////////////

module assemble() {
//batt_frame starts at 0 height, move everything to origin

//add -.1 offset to look nicer in preview mode
translate ([0,0,-.1])    
    
difference() {    
color("midnightblue") translate([-solar_panel_length/2, -solar_panel_width/2, -solar_panel_thickness]) solar_panel();
    
color("white") translate([215, -30, -solar_panel_thickness - 6]) 
    linear_extrude(height = 10) {
mirror ([1,0,0]) text("TRILLSAT", font = "Liberation Sans", size = 70, direction = "ltr", spacing = 1 );
}
}
    
color("silver") translate([-solar_panel_length/2, -solar_panel_width/2, 0]) aluminum_sheet();
 
 if (showpart == 102) translate ([0,0,20])  
     //add .1 offsets to look nicer in preview mode
 translate ([0,-.1,0])       
color("slategray") translate([-batt_box_length/2,-solar_panel_mount_width/2, 0]) batt_frame();
 else  translate ([0,-.1,0])       
color("slategray") translate([-batt_box_length/2,-solar_panel_mount_width/2, 0]) batt_frame();
    
if (showpart == 101) color("dodgerblue") translate([-batt_box_length/2,-solar_panel_mount_width/2, 0]) rope();   
    
 if (showpart == 102) translate ([0,0,130])  
    
color("white") translate([-max_gondola_length/2, -max_gondola_width/2, batt_thickness + motor_mount_depth]) gondola();
 else color("white") translate([-max_gondola_length/2, -max_gondola_width/2, batt_thickness + motor_mount_depth]) gondola();
    
   if (showpart == 102)  translate([0,0,130])
   translate ([0,0,150]) magnet_holder(); 
   else    translate ([0,0,150]) magnet_holder(); 
 
 //ldr mount   
   if (showpart == 102) translate ([0,0,170]) 
color("white") translate ([0,44,78]) mirror ([0,1,0]) mirror ([0,0,1]) rotate ([45,0,0]) ldr_mount();
   else color("white") translate ([0,44,78]) mirror ([0,1,0]) mirror ([0,0,1]) rotate ([45,0,0]) ldr_mount();
    
     //ldr mount  
     if (showpart == 102) translate ([0,0,170])  
    color ("darkgray") translate ([0,-44,78])  mirror ([0,0,1]) rotate ([45,0,0]) ldr_mount();
     else  color ("darkgray") translate ([0,-44,78])  mirror ([0,0,1]) rotate ([45,0,0]) ldr_mount();
    
 if (showpart == 102) translate ([0,0,130])    
color("white") translate([0, 0, 0]) gyro();
 else color("white") translate([0, 0, 0]) gyro();

 if (showpart == 102) translate ([0,0,170])        
color("darkgrey") mirror ([0,1,0]) translate([0,0,batt_thickness + motor_mount_depth + base_thickness+screwdriver_length + endcap_drip_edge + endcap_height + endcap_hall_height]) mirror ([0,0,1]) end_cap(); 
 else
     color("darkgrey") mirror ([0,1,0]) translate([0,0,batt_thickness + motor_mount_depth + base_thickness+screwdriver_length + endcap_drip_edge + endcap_height + endcap_hall_height]) mirror ([0,0,1]) end_cap(); 

  
 if (showpart == 102) translate ([0,0,210])      
color("grey") rotate([0,0,180]) translate([0,0,batt_thickness + motor_mount_depth + base_thickness+screwdriver_length + endcap_drip_edge + endcap_height + endcap_hall_height]) pulley_frame();    
 else
     color("grey") rotate([0,0,180]) translate([0,0,batt_thickness + motor_mount_depth + base_thickness+screwdriver_length + endcap_drip_edge + endcap_height + endcap_hall_height]) pulley_frame();    


 if (showpart == 102) translate ([0,0,260])       
color("white") mirror ([1,0,0]) translate([0,0,batt_thickness + motor_mount_depth + base_thickness+screwdriver_length + endcap_drip_edge + endcap_height + endcap_hall_height +4]) rotate ([0, 0, -90]) mirror ([0,1,0]) capstan();   
else 
    color("white") mirror ([1,0,0]) translate([0,0,batt_thickness + motor_mount_depth + base_thickness+screwdriver_length + endcap_drip_edge + endcap_height + endcap_hall_height +4]) rotate ([0, 0, -90]) mirror ([0,1,0]) capstan();  
   
    //display only, not to be printed
    if (showpart == 100 || showpart == 101)
   color("silver") mirror ([1,0,0]) translate([0,0,batt_thickness + motor_mount_depth + base_thickness+screwdriver_length + endcap_drip_edge + endcap_height + endcap_hall_height + 4]) rotate ([0, 0, -90]) capstan_rod();  
    
    if (showpart == 102) translate ([0,0,250])    
   color("silver") mirror ([1,0,0]) translate([0,0,batt_thickness + motor_mount_depth + base_thickness+screwdriver_length + endcap_drip_edge + endcap_height + endcap_hall_height + 4]) rotate ([0, 0, -90]) capstan_rod();  
    
   if (showpart == 200) rotate ([0,0,$t*360])       
   color("silver") mirror ([1,0,0]) translate([0,0,batt_thickness + motor_mount_depth + base_thickness+screwdriver_length + endcap_drip_edge + endcap_height + endcap_hall_height + 4]) rotate ([0, 0, -90]) capstan_rod();  

    
   // add .1 offsets to look nicer in preview mode
     if (showpart == 102) translate ([-.1,.1,25])   
  color("slategray") translate([solar_panel_length/2 - circuit_box_length,-circuit_box_width/2, brace_height]) radio_box();
     else
           translate ([-.1,.1,0]) color("slategray") translate([solar_panel_length/2 - circuit_box_length,-circuit_box_width/2, brace_height]) radio_box();
  
  //radio box cover - display only
   
     if (showpart == 100 || showpart == 101) color("silver") translate([solar_panel_length/2 - circuit_box_length,-circuit_box_width/2 + circuit_box_thickness, brace_height + circuit_box_height - circuit_box_drip_edge]) cube([124,circuit_box_width - circuit_box_thickness * 2,2]);
        
   
    //add .1 offsets to look nicer in preview mode
  if (showpart == 102) translate ([-.1,.1,25])    
color("slategray") translate([-solar_panel_length/2,-circuit_box_width/2, brace_height]) switching_box();
  else
      translate ([-.1,.1,0])  color("slategray") translate([-solar_panel_length/2,-circuit_box_width/2, brace_height]) switching_box();
   
     //switching box cover - display only
  
    if (showpart == 100 || showpart == 101) color("silver") translate([-solar_panel_length/2 + 70,-circuit_box_width/2 + circuit_box_thickness, brace_height + circuit_box_height - circuit_box_drip_edge]) cube([90,circuit_box_width - circuit_box_thickness * 2,2]);
  

  if (showpart == 102) translate ([0,0,75])    
color("white") translate([-solar_panel_length/2 - trim_thickness/2 + thin_trim_thickness,61, brace_height]) rotate ([0,0,-90]) cpu_holder();
  else
      color("white") translate([-solar_panel_length/2 - trim_thickness/2 + thin_trim_thickness,61, brace_height]) rotate ([0,0,-90]) cpu_holder();

  if (showpart == 102) translate ([0,0,100])    
color("darkgrey") translate([-solar_panel_length/2 - trim_thickness/2 + thin_trim_thickness + 72 + circuit_box_thickness,-circuit_box_width/2, brace_height + 58]) mirror ([1,0,0]) mirror ([0,0,1]) qi_holder();
  else
      color("darkgrey") translate([-solar_panel_length/2 - trim_thickness/2 + thin_trim_thickness + 72 + circuit_box_thickness,-circuit_box_width/2, brace_height + 58]) mirror ([1,0,0]) mirror ([0,0,1]) qi_holder();

color("white") translate([solar_panel_length/2 + thin_trim_thickness, -solar_panel_width/2 - trim_thickness/2, - solar_panel_thickness - thin_trim_thickness]) rotate(a=[0,-90,0])  corner_trim();

color("white") mirror ([1,0,0]) translate([solar_panel_length/2 + thin_trim_thickness, -solar_panel_width/2 - trim_thickness/2, - solar_panel_thickness - thin_trim_thickness]) rotate(a=[0,-90,0])  corner_trim();
    
color("white") translate([0, -solar_panel_width/2 -trim_thickness/2, brace_height + thin_trim_height]) rotate(a=[-90,0,0]) side_trim_right_wire();  
    
color("white") translate([-max_print_width, -solar_panel_width/2 -trim_thickness/2, brace_height + thin_trim_height]) rotate(a=[-90,0,0]) side_trim_left_wire();  
    
color("white") translate([max_print_width, solar_panel_width/2 + trim_thickness/2, brace_height + thin_trim_height]) rotate(a=[-90,0,180]) side_trim_left_wire();  
    
        
color("white") translate([0, solar_panel_width/2 + trim_thickness/2, brace_height + thin_trim_height]) rotate(a=[-90,0,180]) side_trim_right_wire();  

//Only the ITSO side is represented for display
color("silver") translate([solar_panel_length/2 - 27.5, solar_panel_width/2 -6.5, brace_height - 3.5]) rotate(a=[-90,0,180]) aluminum_bar_template(); 
 
 //Only the ITSO side is represented for display
color("silver") translate([-solar_panel_length/2 + 27.5, solar_panel_width/2 -6.5, brace_height - 3.5]) rotate(a=[-90,0,180]) mirror ([1,0,0]) aluminum_bar_template();  

 color("white") translate([solar_panel_length/2 - circuit_box_length - brace_flange_thickness, -solar_panel_mount_width/2,0]) rotate(a=[90,0,90]) brace();   
color("white") mirror ([1,0,0]) translate([solar_panel_length/2 - circuit_box_length - brace_flange_thickness, -solar_panel_mount_width/2,0]) rotate(a=[90,0,90]) brace(); 
    
      if (showpart == 102) translate ([0,0,50])     
color("white") translate([solar_panel_length/2 - circuit_box_length/2 + 38, -circuit_box_width/2 + circuit_box_thickness,brace_height + circuit_box_height - circuit_box_thickness * 2 - circuit_box_drip_edge + .1]) acrylic_separator(); 
      else
          color("white") translate([solar_panel_length/2 - circuit_box_length/2 + 38, -circuit_box_width/2 + circuit_box_thickness,brace_height + circuit_box_height - circuit_box_thickness * 2 - circuit_box_drip_edge + .1]) acrylic_separator(); 

//rotate -2.5 degrees on 3rd value to simulate line up with qi coil, assumes rod/screwdriver shaft bends 2 degrees under weight, so hole won't line up with rod exactly - had to line up everything manually

//note that rod and bb hole subtractions are commented
//out in planet() module when doing animation
//since they don't match up
  if (showpart == 100 || showpart == 101) color("grey") mirror ([1,0,0]) rotate ([0,0,0]) translate ([-243,0,73.5 + 0]) planet();
  if (showpart == 102) translate ([0,0,245]) color("grey") mirror ([1,0,0]) rotate ([0,0,0]) translate ([-243,0,73.5 + 0]) planet();    
  if (showpart == 200) rotate ([0,0,$t*360]) color("grey") mirror ([1,0,0]) rotate ([0,0,0]) translate ([-243,0,73.5 + 0]) planet();
}
  
////////////////////////////// PARTS ////////////////////////////////


module solar_panel () {
 cube([solar_panel_length, solar_panel_width, solar_panel_thickness]); 
    

}

module aluminum_sheet () {
cube([solar_panel_length, solar_panel_width, .1]); 
}

module corner_trim () {
difference() {
corner_trim_base ();
corner_trim_cuts ();    
}
}




module side_trim_right_wire () {
difference () {    
  difference() {
    side_trim_base ();
    side_trim_cuts ();    
  }
  side_trim_middle_slot();
  side_trim_wire_slot_right();
  side_trim_box_cut_right();
  side_trim_right_mounting_holes();
}
}

module side_trim_left_wire () {
difference () {    
  difference() {
    side_trim_base ();
    side_trim_cuts ();    
  }
  side_trim_middle_slot();
  side_trim_wire_slot_left();
  side_trim_box_cut_left();
  side_trim_left_mounting_holes();
}
}

module brace () {
difference() {
brace_base ();
brace_cuts ();    
}
}

module batt_frame () {
 batt_box(); 
}    

//rope is at a 12 degree angle
module rope() {
     translate ([-9, solar_panel_mount_width/2, 16]) rotate ([0,12,0]) cylinder ( 170, 5/2, 5/2);
     translate ([batt_box_length + 9, solar_panel_mount_width/2, 16]) rotate ([0,-12,0]) cylinder ( 185, 5/2, 5/2);
    translate ([-280 + batt_box_length/2,solar_panel_mount_width/2, brace_height/2]) rotate ([0,90,0]) cylinder (560, 5/2, 5/2);
}

module gondola () {
   difference() { 
    base_and_fins();
    cut_holes();
   }
}

module end_cap () {
    difference() {
        cap_base();
        cap_holes();
    }   
}

module pulley_frame () {
    difference() {
        pulley_frame_base();
        pulley_frame_holes();
    }  
}

module ldr_mount () {
    difference() {
        ldr_base();
        ldr_holes();
    }  
}

module magnet_holder () {
    difference() {
        magnet_holder_base();
        magnet_holder_holes();
    }  
}

module qi_holder () {
    
    //lower entire assembly to test
    //difference() {
   // translate ([0,0,-1]) {
    
    difference() {
        qi_holder_base();
        qi_holder_holes();
    }  
    difference() {
        //additions/subtractions removed to see through for planet calibration - re-enable later
      qi_holder_additions();
      qi_holder_subtractions();
    }    
    
//}

   //remove anything under the bottom
   // translate ([-10,-10,-10]) cube ([100,200, 10]);
//}
    
}


module hex_nut_calibrator() {
    difference() {
        hex_nut_base();
        hex_nut_holes();
    }   
    
}


////////////////// MAGNET HOLDER ////////////
module magnet_holder_base() {
cylinder (magnet_holder_height,screwdriver_shaft_diameter/2 + magnet_holder_thickness, screwdriver_shaft_diameter/2 + magnet_holder_thickness);
}

module magnet_holder_holes() {
translate ([0,0,-.1]) cylinder ( 99, screwdriver_shaft_diameter/2 +magnet_holder_looseness, screwdriver_shaft_diameter/2 + magnet_holder_looseness);
//rare earth magnet (slightly larger)
translate ([screwdriver_shaft_diameter/2 + .15,0,magnet_holder_height/2]) rotate ([0,90,0]) cylinder (99, magnet_diameter/2 + .15, magnet_diameter/2 + .1);
//regular magnet
translate ([0,screwdriver_shaft_diameter/2 + .15,magnet_holder_height/2]) rotate ([-90,0,0]) cylinder (99, magnet_diameter/2 - .05, magnet_diameter/2 - .05);
}

/////////////// QI HOLDER ////////////////////////

qi_offset = 12;
module qi_holder_base() {
 
    //remove -35
translate ([0,circuit_box_thickness, 0]) cube([72, circuit_box_width - circuit_box_thickness * 2 , 20]);  
    
 //long velcro holders   
  translate ([-7 ,(circuit_box_width - circuit_box_thickness * 2)/2 - 25,1.8]) rotate ([-90,0,0]) cylinder (50, 3.5, 3.5);
    
  translate ([72 + 7,(circuit_box_width - circuit_box_thickness * 2)/2 - 25,1.8]) rotate ([-90,0,0]) cylinder (50, 3.5, 3.5);
    
// square braces to attach velcro holders to docking pad
  translate ([ -7, 52, 0]) cube ([ 12, 7- 1.8, 7- 1.8]);    
    
  translate ([ -7, (circuit_box_width - circuit_box_thickness * 2) - 52 - (7-1.8), 0]) cube ([ 12, 7- 1.8, 7- 1.8]);  
    
      translate ([ 72 + 7 - 12, 52, 0]) cube ([ 12, 7- 1.8, 7- 1.8]);    
    
  translate ([ 72 + 7 - 12, (circuit_box_width - circuit_box_thickness * 2) - 52 - (7-1.8), 0]) cube ([ 12, 7- 1.8, 7- 1.8]);

}


module qi_holder_holes() {
    
   // translate ([36, (circuit_box_width - circuit_box_thickness * 2)/2 + qi_offset, 1.5]) cylinder (99.9, 22, 22);
    
  //test - made 1.5 negative to protrude so I can see it for adjusting planet, change to positive for normal  
    translate ([36, (circuit_box_width - circuit_box_thickness * 2)/2 + qi_offset, 1.5]) cylinder (99.9, 25.2, 25.2);
    

    //circuit board assembly
    translate ([31,0,0]) rotate ([ 0,0,49]) {
       //circuit board inset
       translate ([32,6, 12]) cube([34, 34, 99.9]);  
        
       //circuit board screw holes 
       translate ([32 + 6.5, 6 + 34 - 12.5,3]) cylinder(99.9,.7,.7); 
       translate ([32 + 13.5, 6 + 11.5,3]) cylinder(99.9,.7,.7); 
       
       //usb wire slot
       translate ([-12, 18, 12]) cube([48, 11, 99.9]);  
        
       //wire and led slot
       translate ([65, 18, 1.5]) cube([13, 11, 99.9]);  
    }    
    
    //screw holes
    translate ([16,4,-.9]) cylinder(99.9,1.4,1.4); 
    translate ([72 - 2,4,-.9]) cylinder(99.9,1.4,1.4); 
    translate ([72 - 2,circuit_box_width - 4,-.9]) cylinder(99.9,1.4,1.4); 
    translate ([16,circuit_box_width - 4,-.9]) cylinder(99.9,1.4,1.4); 
    
    //screw head inset holes with coned/tapered tops
    translate ([16,4,0]) cylinder(15,2.4,2.4); 
    translate ([16,4,15]) cylinder(2.5,2.4,1.4); 
    
    
    translate ([72 - 2,4,0]) cylinder(15,2.4,2.4); 
    translate ([72 - 2,4,15]) cylinder(2.5,2.4,1.4); 
    
    translate ([72 - 2,circuit_box_width - 4,0]) cylinder(15,2.4,2.4); 
    translate ([72 - 2,circuit_box_width - 4,15]) cylinder(2.5,2.4,1.4); 
    
    
    translate ([16,circuit_box_width - 4,0]) cylinder(15,2.4,2.4); 
    translate ([16,circuit_box_width - 4,15]) cylinder(2.5,2.4,1.4); 
    

    
    //side slope
    translate ([-9.9,circuit_box_thickness + 3, -20]) rotate ([45,0,0]) cube([99.9, 20, 70]);  
    
     translate ([-9.9,113, -20]) rotate ([-45,0,0]) cube([99.9, 20, 70]);  
    
    //velcro holder side slopes
    translate ([-20,41, -20]) rotate ([45,0,0]) cube([20, 20, 70]);  
    translate ([-20,73.5, -20]) rotate ([-45,0,0]) cube([20, 20, 70]);  
    
    translate ([72,41, -20]) rotate ([45,0,0]) cube([20, 20, 70]);  
    translate ([72,73.5, -20]) rotate ([-45,0,0]) cube([20, 20, 70]);  
    
    //text to make it lighter weight
    translate ([3,123,18]) {
linear_extrude(height = 99.9) {
text("TRILL", font = "Liberation Sans", size = 15, direction = "ltr", spacing = 1 );
}}

    translate ([5,110,18]) {
linear_extrude(height = 99.9) {
text("SAT-1", font = "Liberation Sans", size = 9, direction = "ltr", spacing = 1 );
}}

    
    //remove anything under the bottom
    translate ([-10,-10,-10]) cube ([100,200, 10]);
}

module qi_holder_additions() {
 translate ([36, (circuit_box_width - circuit_box_thickness * 2)/2 + qi_offset, 1.5]) cylinder (1.5, 9.2, 9.2);
translate ([36, (circuit_box_width - circuit_box_thickness * 2)/2 + qi_offset, 1.5]) cylinder (2.5, 2, 2);
 
}

module qi_holder_subtractions() {
       //coil screw hole
       translate ([36, (circuit_box_width - circuit_box_thickness * 2)/2 + qi_offset, -.9]) cylinder (99.9, 1, 1);
    

 
}



module hex_nut_base() {
cube ([25, 33, 3]);    
    
    
}

module hex_nut_holes() {

//8-32 nut - small - a little loose, .2 is probably PERFECT
translate ([6, 6 + 21, .5]) cylinder(9.9, hex_diameter_8_32/2 + .25, hex_diameter_8_32/2 + .25, $fn=6); 
    
//8-32 nut - medium
translate ([6, 6 + 10.5, .5]) cylinder(9.9, hex_diameter_8_32/2 + .3, hex_diameter_8_32/2 + .3, $fn=6);
    
//8-32 nut - large
translate ([6, 6, .5]) cylinder(9.9, hex_diameter_8_32/2 + .35, hex_diameter_8_32/2 + .35, $fn=6);
    
//6-32 net - small
translate ([18, 6 + 21, .5]) cylinder(9.9, hex_diameter_6_32/2 + .05, hex_diameter_6_32/2 + .05, $fn=6);

//6-32 net - medium - PERFECT
translate ([18, 6 + 11, .5]) cylinder(9.9, hex_diameter_6_32/2 + .1, hex_diameter_6_32/2 + .1, $fn=6);
  
//6-32 net - large
translate ([18, 6, .5]) cylinder(9.9, hex_diameter_6_32/2 + .15, hex_diameter_6_32/2 + .15, $fn=6);
    
}    

module aluminum_bar_template() {
    difference() {
        aluminum_bar_template_base();
        aluminum_bar_template_holes();
    }   
    
}

module aluminum_bar_template_base() {
cube ([85, 10, 1.5]);       
}

module aluminum_bar_template_holes() {
         //aluminum bar
         translate ([7.5, 4,-.9])    cylinder (99.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1); 
         translate ([7.5 + 15, 4,-.9])    cylinder (99.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1); 
    
          translate ([58 + 7.5, 4,-.9])    cylinder (99.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1); 
         translate ([58 + 7.5 + 15, 4,-.9])    cylinder (99.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1); 
}

module side_trim_template() {
    difference() {
    side_trim_right_wire();
    translate ([- 500,-500,1.5]) cube ([999,999,99.9]);
    }
}

module corner_trim_template() {
    difference() {
    rotate ([90,0,0]) corner_trim();
    translate ([- 500,-500,1.5]) cube ([999,999,999]);
    }
}

module gyro () {

    fan_hub_radius = gyro_mount_diameter/2 + gyro_center_ring_thickness;
    fan_tip_radius = gyro_disc_diameter/2 + shot_diameter + 2;
    optimum_blade_number = (6 * (fan_hub_radius / fan_tip_radius)) / 1 - ( fan_hub_radius / fan_tip_radius);
    echo ("optimum blade number = ", optimum_blade_number);
    
    difference() {
    gyro_base();
    gyro_cutouts();
    }

    

 
}    

module capstan () {
    
    difference() {
        capstan_base();
        capstan_holes();
    }   
}

module planet () {
    
    difference() {
        planet_base();
        //cut out a section to see inside
        //translate ([-50,-110,-1]) cube ([100,100,150]);
    }   
}




////////////// PULLEY BRACKET SLAB ////////////
    pulley_bracket_length = 15;
    module pulley_bracket_slab() {
       //curved end
       translate ([5/2 - 1.5, 18/2, 13/2 + 5/2 + 2]) rotate ([90, 0, 0]) cylinder (18, 11, 11); 
        //rectangle slab - shift 2 mm to make rain cover and make larger by 2 mm
       translate ([-bearing_outer_diameter/2 - 3.5, 18/2, 11]) rotate ([90, 0, 0]) cube ([22, pulley_bracket_length - 5/2 - 1 + 5, 18]);
        
      // second rectangle to make second rope guide (comment out when printing gondola pulleys or batt frame)
        if (showpart != 2 && showpart != 6) {
       translate ([-bearing_outer_diameter/2 - 3.5, 18/2,0]) rotate ([90, 0, 0]) cube ([11, pulley_bracket_length - 5/2 - 1 + 5, 18]); 
        
     //rectangle angled to slope to rope guide (comment out when printing gondola pulleys)
       translate ([-14.8,-9,2]) rotate ([0,63.2,0]) cube ([10,18,17]);
        }

    
    
    }

////////////// PULLEY BRACKET ///////////
module pulley_bracket () {
      difference() {
         pulley_bracket_base();
         pulley_bracket_holes();
      }  
  
module pulley_bracket_base () {
      difference() {
         pulley_bracket_slab();
         pulley_bracket_slot();
      }  
      
      //bearing ring
      translate ([0, -3, 13/2 + 2]) rotate ([90, 0, 0]) cylinder (1, 6/2, 6/2); 
      //mirror bearing ring to other side
      mirror ([0,1,0]) translate ([0, -3, 13/2 + 2]) rotate ([90, 0, 0]) cylinder (1, 6/2, 6/2); 
      
      
         //rope guide posts (comment out for batt frame)
      if (showpart != 6) {
        translate ([-7.5,-4,-4.8]) cylinder (10, 1.5, 1.5); 
        mirror ([0,1,0]) translate ([-7.5,-4,-4.8]) cylinder (10, 1.5, 1.5); 
      }
  }  
    
    module pulley_bracket_slot() {
       //slot for pulley, shift 1mm to allow rain cover
        translate ([-bearing_outer_diameter/2 -4.1 + .5, - (bearing_width + 2)/2, -.1]) cube ([99.9, bearing_width + 2, bearing_outer_diameter + 5]);
        
       // third rectangle to make second rope guide
       translate ([-bearing_outer_diameter/2 - 3.5 - .1, (bearing_width + 2)/2,-7 - .1]) rotate ([90, 0, 0]) cube ([99.9, 10, bearing_width+2]);
        
    }    

    module pulley_bracket_holes() {

       //pulley axel hole
       translate ([0, 18/2 + .1, 13/2 + 2]) rotate ([90, 0, 0]) cylinder (99.9, bearing_inner_diameter/2 + .1, bearing_inner_diameter/2 + .1); 
    
      //rope guide hole (2mm higher than bearing for taught paracord)
     translate ([bearing_outer_diameter/2, 0, ])cylinder (99.9, 5/2, 5/2); 
    
    //disable venturis when printing batt_frame   
    if (showpart != 6) {  
    translate ([bearing_outer_diameter/2, 0, 17.4 ]) rope_hole_venturi();
    translate ([bearing_outer_diameter/2, 0, 28 ]) mirror ([0,0,1])  rope_hole_venturi();  
    }
    
        
        //erase side protrusion
        translate ([-20,-10,-9.1])  cube ([10,20,17]);
    
        
    }   
    
    module rope_hole_venturi() {
    difference() {  
       cylinder (2.5, (bearing_width + 2)/2,(bearing_width + 2)/2);  
     translate ([0,0,2.5]) rotate_extrude(convexity = 10) translate([(bearing_width + 2)/2, 0, 0]) circle(r = 2);  
}
}

      //translate ([bearing_outer_diameter/2, 0, 15.5 ]) rope_hole_venturi();
 

    
}



///////////////////////// PULLEY FRAME /////////////////////////
module pulley_frame_base () {
  //top of endcap
  cylinder(2.5,screwdriver_orange_length/2 + cylinder_thickness,screwdriver_orange_length/2 + cylinder_thickness); 
    
  //cone
  difference () {
    translate ([0,0,2.5]) cylinder(40,screwdriver_orange_length/2 + cylinder_thickness,screwdriver_orange_length/2 + cylinder_thickness - 10); 
    
    //cut off half of frame, tilt and shift to right
    //rotate ([0,-30,24]) translate ([screwdriver_orange_length/2 + cylinder_thickness,0,-40]) cylinder (100,screwdriver_orange_length/2 + cylinder_thickness,screwdriver_orange_length/2 + cylinder_thickness);

    //create plateau
    translate ([-50,-50,42]) cube ([100,100,100]);
      
      //subtract slabs to create space for pulley bracket
           translate ([-40,0,28.5]) rotate ([0,-90,180]) rotate ([-12,0,-5]) pulley_bracket_slab();
           translate ([0,0,-21.5]) mirror ([1,0,0]) translate ([-47,0,30]) rotate ([0,-90,180]) rotate ([-12,0,-5]) pulley_bracket_slab();
}

  module pulley_frame_high_bracket() {
      pulley_bracket();
      //rectangle support for print overhang
       difference () {
        translate ([-30,-9,-6.7]) rotate ([0,0,0]) cube ([20,18,34.4]);
        translate ([-30,-10,7.2]) rotate ([0,125,0]) cube ([17,20,28]);
        translate ([-29.9,-(bearing_width + 2)/2,-6.7]) rotate ([0,0,0]) cube ([20,bearing_width + 2,24.4]);
       }
       
  }   

//angle toward capstan at 12 degrees, toward base on different axis at 5 degrees
  translate ([-40,0,28.5]) rotate ([0,-90,180]) rotate ([-12,0,-5])  pulley_frame_high_bracket();
  translate ([0,0,-21.5]) mirror ([1,0,0]) translate ([-47,0,30]) rotate ([0,-90,180]) rotate ([-12,0,-5]) pulley_bracket();


     
}


module pulley_frame_holes () {
     
   //screwdriver shaft hole
   translate ([0,0,-9.9]) cylinder(99.9,capstan_diameter/2 + 2, capstan_diameter/2 + 2); 
    
         //left endcap screw holes
     translate ([-screwdriver_orange_length/2 + 1, - 15,-.1]) cylinder(99,outer_diameter_6_32/2 + .2,outer_diameter_6_32/2 + .2);  
    
          translate ([-screwdriver_orange_length/2 + 1,15,-.1]) cylinder(99,outer_diameter_6_32/2 + .2,outer_diameter_6_32/2 + .2);  
    
        //right endcap screw holes
          translate ([screwdriver_orange_length/2 - 1,15,-.1]) cylinder(99,outer_diameter_6_32/2 + .2,outer_diameter_6_32/2 + .2);  

      translate ([screwdriver_orange_length/2 - 1,-15,-.1]) cylinder(99,outer_diameter_6_32/2 + .2,outer_diameter_6_32/2 + .2);  
    
        //temp probe hole
    translate ([0,18.5,-.9]) cylinder(100, temp_probe_diameter/2+ 2.5 + 1.5, temp_probe_diameter/2+ 2.5 + 1.5);
    
    //left endcap inset for screw heads
    
         translate ([-screwdriver_orange_length/2 + 1, - 15,2.5]) cylinder(99,outer_diameter_6_32/2 + 2,outer_diameter_6_32/2 + 2);  
    
          translate ([-screwdriver_orange_length/2 + 1,15,2.5]) cylinder(99,outer_diameter_6_32/2 + 2,outer_diameter_6_32/2 + 2);  
          
        //right endcap inset for screw heads
          translate ([screwdriver_orange_length/2 - 1,15,2.5]) cylinder(99,outer_diameter_6_32/2 + 2,outer_diameter_6_32/2 + 2);  

      translate ([screwdriver_orange_length/2 - 1,-15,2.5]) cylinder(99,outer_diameter_6_32/2 + 2,outer_diameter_6_32/2 + 2);  
    
        //temp probe hole
    translate ([0,18.5,-.9]) cylinder(100, temp_probe_diameter/2+ 2.5 + 1.5, temp_probe_diameter/2+ 2.5 + 1.5);
    
    //remove bottom protrusions
    translate ([-100,-100,-10]) cube ([200,200,10]);
    

    

}

/////////////////////////  CAPSTAN /////////////////////////////
    
 module capstan_base() {
     
difference() {  
cylinder (15, capstan_diameter/2, capstan_diameter/2);  
translate ([0,0,15]) rotate_extrude(convexity = 10) translate([capstan_diameter/2, 0, 0]) circle(r = 10);  
}

//middle cylinder
translate ([0, 0, 15]) cylinder (6, 9, 9);  

//middle flange
  translate ([0,0, 6 + 15]) 
    difference() {
        cylinder (9, capstan_diameter/2, capstan_diameter/2);  
        rotate_extrude(convexity = 10) translate([capstan_diameter/2, 0, 0]) circle(r = 10);  
        }


//middle cone
    translate ([0, 0, 6 + 24]) cylinder (22, 15, capstan_diameter/2 + 7);
    


//top cone
translate ([0, 0, 52]) cylinder (26, capstan_diameter/2 + 7, 2);  

 }

module capstan_holes() {
capstan_rod();
 
//screwdriver shaft hole
//translate ([0, 0, -.1]) cylinder(1.1,endcap_shaft_hole/2, endcap_shaft_hole/2);  
    
//hex shaft hole
translate ([0, 0, -.1]) cylinder(37, 3.9, 3.9 , $fn=6); 
} 

///////////////////// CAPSTAN ROD /////////////////////////
 module capstan_rod() {
 //is make a little bigger to allow a 1/4 inch rod to fit
mirror ([0,0,0]) translate ([0, 10, 64]) rotate ([180 - 62,0,0]) {
     
     cylinder (300.5, capstan_rod_diameter/2 + .3, capstan_rod_diameter/2 + .3); 

 }
     
 }

/////////////////////////// PLANET /////////////////////////////
module planet_base(){
       
    translate ([0,0,20 + 10]) difference() {
      
      //comment out sphere to see inside void shape  
      translate ([0,0,-1]) sphere (40);
      planet_hole_shape();
    }
}

module planet_hole_shape_cone() {
                
  difference () {
          sphere (37); 
        //flatten top of sphere
          translate ([0,0,0]) cylinder (37, 38, 38);
       //add plane to flatten bottom, cut off 10 mm
       translate ([-50,-50,-40]) cube ([100,100,10 + 2]);
   } 
   translate ([0,0,-.1]) cylinder (30,37,0); 
           
}
module planet_hole_shape () {
    
  difference() {
       planet_hole_shape_cone();
   
      // rotate ([0,-(90 - 62 - 2),0]) translate ([-50,-50,-40]) cube ([100,100,36]);
      
      //slot to keep brace for rod
      translate ([21.5,-(capstan_rod_diameter + 8)/2,-40]) rotate ([0,-27,0]) cube ([100, capstan_rod_diameter + 8, 100]);
   }
   
  //remove rod and bb hole subtractions
   //when getting screenshots or animations
    if (showpart < 200) {
   //add rod subtraction
   translate ([0,0,12]) rotate ([0,62 + 1,0]) translate ([0,0,-2]) cylinder (50, capstan_rod_diameter/2 + .3, capstan_rod_diameter/2 + .3);
   
      //add bb hole subtraction, end stop, goes past center since rod brace is angled
   translate ([0,0,12]) rotate ([0,62 + 1,0]) translate ([0,0,-9.9]) cylinder (50, bb_diameter/2, bb_diameter/2);
   }
   //add vertical slot for glue to hold rod stable
      translate ([2.5,-(capstan_rod_diameter + 8)/2 + 6.2,5]) rotate ([0,-27,0]) cube ([10, 2, 14]);
  
   //add plane to flatten bottom, cut off 10 mm
   translate ([-50,-50,-42]) cube ([100,100,10 + 2]);
   
}

/////////////////////////  GYRO  ////////////////////////////////

module gyro_base (){
//center disc
    cylinder(5.5,gyro_mount_diameter/2 + gyro_center_ring_thickness, gyro_mount_diameter/2 + gyro_center_ring_thickness);   
//outer ring
 gyro_ring();

    
//12 fan blades  
  for (i=[3.8:30:360])  
 rotate ([0,0,i]) translate ([gyro_mount_diameter/2 + gyro_center_ring_thickness - 2.5, 0, -1.4]) rotate ([35,0,0]) render () {cube ([gyro_disc_diameter/2 - gyro_mount_diameter/2 - gyro_center_ring_thickness - shot_diameter - 1 + 2, 1.8, 7]);}
    
}

module gyro_cutouts() {
    // hole in center of disc
    translate ([0,0,-.1]) cylinder(9.9,gyro_hole_diameter/2 + .2, gyro_hole_diameter/2 + .2);   
    
    //center ring inset for mount
       translate ([0, 0, 2.5]) cylinder(9.9,gyro_mount_diameter/2 + .2, gyro_mount_diameter/2 + .2);   


for (i=[7.5:15:360])
rotate ([0,0,i]) {
    

//smaller cylinder (sideways cone) entry to sphere - allow snap fit
  translate ([gyro_mount_diameter + gyro_center_ring_thickness + 10.3, 0,gyro_thickness + shot_diameter/2 + 1.5]) rotate ([0, 90, 0]) render () {cylinder(5,shot_diameter/2 + .25, shot_diameter/2 - .4);}  
  
  //pin holes for tool to pop out spheres
translate ([gyro_diameter/2 - 2 - shot_diameter/2 - 2,0,gyro_thickness + shot_diameter - 2]) render () {cylinder (9.9, 1.2, 1.2);}
  
 //shot sphere, all the way up to the base
 translate ([gyro_disc_diameter/2 - shot_diameter/2 - 4, 0, shot_diameter/2 + gyro_thickness + 1.5]) 
  
  render () {sphere (shot_diameter/2);}}
  

  
  //tiny screw holes
for (i=[15:120:360])
rotate ([0,0,i]) 
translate ([12,0,-.9]) cylinder (9.9, 1, 1);

//remove any fan blade protrusions from bottom
translate ([0,0,-99.9]) cylinder (99.9, gyro_disc_diameter/2 + 9.9, gyro_disc_diameter/2 + 9.9);

}


module gyro_ring() {
    
 difference () {   
   cylinder(shot_diameter + 3,gyro_disc_diameter/2, gyro_disc_diameter/2);  

//just enough to keep sphere inside ring
   translate ([0,0,-.1]) cylinder(99.9,gyro_disc_diameter/2 - shot_diameter - 4, gyro_disc_diameter/2 - shot_diameter - 4); 
 }
    
}


///////////////////////// LDR MOUNTS /////////////////////////
module ldr_base() {

//main tube
cylinder(ldr_tube_length,ldr_tube_diameter/2, ldr_tube_diameter/2);   
    
//mount flanges
translate ([-(fin_thickness + .1 + ldr_tube_thickness * 2)/2,ldr_tube_diameter/2 - 2,]) ldr_flange ();
    
}

module ldr_holes() {

//main tube hole (raised to level of hole)
translate ([0,0,ldr_tube_thickness + 2]) cylinder(99.9,ldr_diameter/2, ldr_diameter/2);  
   
//lens inset (assumes acrylic is about 2mm thick)
translate ([0,0,ldr_tube_length - 2]) cylinder(99.9,ldr_lens_diameter/2, ldr_lens_diameter/2 - ldr_tube_thickness/2);  
   
   
    //wire hole
translate ([0,99.9,ldr_tube_thickness + 3.3]) rotate ([90,0,0]) cylinder(99.9,1.2, 1.2);  
    
    //screw hole    
translate ([-9.9,17,ldr_tube_thickness + 21]) rotate ([0,90,0]) cylinder(99.9,outer_diameter_6_32/2 + .2, outer_diameter_6_32/2 + .2); 
}

 

module ldr_flange() {
    difference() {

        ldr_flange_base();
        translate ([ldr_tube_thickness - .6,4,-.1]) cube([fin_thickness + 1.2, 99.9,99.9]); 
    }
}   

module ldr_flange_base () {
    //square offset of base
    translate ([0,0,0]) rotate ([45,0,0]) cube ([fin_thickness + ldr_tube_thickness * 2, 27.5, ldr_tube_length -8]);
      
    
}



///////////////////////// END CAP /////////////////////////
module cap_base () {

//top of endcap
cylinder(endcap_height + endcap_hall_height,screwdriver_orange_length/2 + cylinder_thickness,screwdriver_orange_length/2 + cylinder_thickness); 

//smaller cylinder that fits inside drip edge (slightly smaller) - uses endcap length * 2 to account for error in gondola drip edge code
//also this double drip edge is made .2mm smaller to make sure cap closes all the way    
translate ([0,0,endcap_height + endcap_hall_height]) cylinder(endcap_drip_edge * 2 - .2,screwdriver_orange_length/2 + 7 - .2,screwdriver_orange_length/2 + 7 - .2); 

//even smaller cylinder that fits inside screwdriver tube (slightly smaller)  has to be drip edge * 2 to account for error in gondola drip edge code
translate ([0,0,endcap_height + endcap_hall_height + endcap_drip_edge * 2 - .2]) cylinder(endcap_cone_depth - 2,screwdriver_orange_length/2 - .2,screwdriver_orange_length/2 - .2); 
    
   //for first capstan size, angle it at 65 degrees instead of 45 degrees (set to 25 degree rotation) 
   //translate ([39, 0, 0]) rotate ([0,0,25]) cap_flange ();
   //translate ([-39, 0, 0]) rotate ([0,0,180 - 25]) cap_flange ();
    
   //translate ([70, 0, 0]) cap_flange ();
    
}

module cap_flange () {
   difference() {
       cap_flange_base();
       cap_flange_holes();
   }    
    //pulley axel hole
   difference () {
    translate ([3.5, -3, 13/2 - 3.5]) rotate ([90, 0, 0]) cylinder (1, 6/2, 6/2); 
    translate ([3.5, 18/2 + .1, 13/2 - 3.5]) rotate ([90, 0, 0]) cylinder (99.9, bearing_inner_diameter/2 + .1, bearing_inner_diameter/2 + .1); 
   }
   difference () {
    translate ([3.5, 4, 13/2 - 3.5]) rotate ([90, 0, 0]) cylinder (1, 6/2, 6/2); 
    translate ([3.5, 18/2 + .1, 13/2 - 3.5]) rotate ([90, 0, 0]) cylinder (99.9, bearing_inner_diameter/2 + .1, bearing_inner_diameter/2 + .1); 
   }
}

module cap_flange_base () {
        translate ([-17, -18/2, 0]) cube ([17, 18, endcap_height + endcap_hall_height]);
        //rounded end
        translate ([0, 18/2, 13/2]) rotate ([90, 0, 0]) cylinder (18, (endcap_height + endcap_hall_height)/2, (endcap_height + endcap_hall_height)/2); 
        //top angled part of flange
         translate ([0, -18/2, 0]) cube ([(endcap_height + endcap_hall_height)/2, 18, (endcap_height + endcap_hall_height)/2]);
    
}

module cap_flange_holes () {
    //slot for pulley
     translate ([-bearing_outer_diameter/2 + 2, - (bearing_width + 2)/2, -.1]) cube ([99.9, bearing_width + 2, 99.9]);
    //pulley axel hole
    translate ([3.5, 18/2 + .1, 13/2 - 3.5]) rotate ([90, 0, 0]) cylinder (99.9, bearing_inner_diameter/2 + .1, bearing_inner_diameter/2 + .1); 
}


module cap_holes () {
  
       // translate ([38, 0, 0]) rotate ([0,0,25]) cap_flange_holes();
       // translate ([-38, 0, 0]) rotate ([0,0,180 - 25]) cap_flange_holes ();
    
   //screwdriver shaft hole
   translate ([0,0,-.9]) cylinder(99.9,endcap_shaft_hole/2, endcap_shaft_hole/2);  
    
   //cone-shaped hole to fit top of screwdriver over the lock ridge 
   // this also extends cone depth my 1 mm, since my calculations were wrong somewhere
   translate ([0,0,endcap_height + endcap_hall_height + endcap_drip_edge  + .1]) cylinder(endcap_cone_depth + 1,endcap_cone_small_diameter/2, endcap_cone_large_diameter/2);  
    
    //temp probe ring
    translate ([0,18.5,temp_probe_diameter - .1]) mirror ([0,0,1]) temp_probe_ring ();
    
    //temp probe cylinder 1 mm from surface    
    translate ([0,18.5,1]) rotate ([0,0,-90]) temp_probe_hole(); 
    
    //hall magnet cylinder
      translate ([0,0,endcap_height]) cylinder(99.9,endcap_magnet_cylinder_diameter/2, endcap_magnet_cylinder_diameter/2);  
    
    //slot for hall sensor 1
   translate ([-endcap_hall_width/2,endcap_magnet_cylinder_diameter/2 - 3 + endcap_hall_thickness,endcap_height]) cube ([endcap_hall_width, 3,99.9]);
    
    //slot for hall sensor 2
   translate ([-endcap_magnet_cylinder_diameter/2 - endcap_hall_thickness,-endcap_hall_width/2, endcap_height]) cube ([3, endcap_hall_width,99.9]);
   
         //endcap screw holes
      translate ([-screwdriver_orange_length/2 + 1, - 15,-.1]) cylinder(99,outer_diameter_6_32/2 + .2,outer_diameter_6_32/2 + .2);  

      translate ([screwdriver_orange_length/2 - 1,15,-.1]) cylinder(99,outer_diameter_6_32/2 + .2,outer_diameter_6_32/2 + .2);  
    
          translate ([-screwdriver_orange_length/2 + 1,15,-.1]) cylinder(99,outer_diameter_6_32/2 + .2,outer_diameter_6_32/2 + .2);  

      translate ([screwdriver_orange_length/2 - 1,-15,-.1]) cylinder(99,outer_diameter_6_32/2 + .2,outer_diameter_6_32/2 + .2);  
      
     
}

module temp_probe_hole () {

  cylinder(99.9,temp_probe_diameter/2,temp_probe_diameter/2);
}

module temp_probe_ring () {
    rotate_extrude(convexity = 10) translate ([temp_probe_diameter/2+ 2.5,0,0]) circle(r = 1.5);
    
    difference() {
    //temp probe cylinder        
cylinder(temp_probe_diameter,temp_probe_diameter/2 + 4,temp_probe_diameter/2 + 4); 
        
            //temp probe cylinder        
translate ([0,0,-.1]) cylinder(99.9,temp_probe_diameter/2 + 1,temp_probe_diameter/2 + 1); 
    }
    
    
}

 
//////////////////////// ACRYLIC SEPARATOR ////////////

module acrylic_separator () {
 cube([acrylic_separator_base_width, circuit_box_width - circuit_box_thickness * 2, circuit_box_thickness]); 
    
 translate ([acrylic_separator_base_width/2 - circuit_box_thickness/2, 0, circuit_box_thickness]) cube([circuit_box_thickness, circuit_box_width - circuit_box_thickness * 2, circuit_box_thickness + circuit_box_drip_edge]); 
    
    translate ([-acrylic_separator_tab_length/2 + acrylic_separator_base_width/2,0,0]) cube([acrylic_separator_tab_length, acrylic_separator_tab_width, circuit_box_thickness]); 
    
        translate ([-acrylic_separator_tab_length/2 + acrylic_separator_base_width/2,circuit_box_width - circuit_box_thickness * 2 - acrylic_separator_tab_width,0]) cube([acrylic_separator_tab_length, acrylic_separator_tab_width, circuit_box_thickness]); 
    
    
}

// a slightly bigger version to create a void
module acrylic_separator_void () {
    acrylic_void_separator_base_width = acrylic_separator_base_width + .2;
    acrylic_void_box_length = circuit_box_width;
    acrylic_void_separator_tab_length = acrylic_separator_tab_length + .2;
    acrylic_void_separator_tab_width =   acrylic_separator_tab_width + .2;
    acrylic_void_thickness = circuit_box_thickness + .2;
    
 cube([acrylic_void_separator_base_width, acrylic_void_box_length -  acrylic_void_thickness * 2,  acrylic_void_thickness]); 
    
 translate ([acrylic_void_separator_base_width/2 - acrylic_void_thickness/2, 0,  acrylic_void_thickness]) cube([acrylic_void_thickness, acrylic_void_box_length -  acrylic_void_thickness * 2,  acrylic_void_thickness + circuit_box_drip_edge]); 
    
    translate ([-acrylic_void_separator_tab_length/2 + acrylic_void_separator_base_width/2,0,0]) cube([acrylic_void_separator_tab_length, acrylic_void_separator_tab_width,  acrylic_void_thickness]); 
    
        translate ([-acrylic_separator_tab_length/2 + acrylic_void_separator_base_width/2,acrylic_void_box_length -  acrylic_void_thickness * 2 - acrylic_void_separator_tab_width,0]) cube([acrylic_void_separator_tab_length, acrylic_void_separator_tab_width,  acrylic_void_thickness]); 
    
    
}

////////////////////////  BRACE ////////////////////////////

module brace_base () {
 cube([solar_panel_mount_width, brace_height, brace_thickness]); 
 
 translate ([0,brace_thickness,0]) cube([solar_panel_mount_width, brace_flange_length, brace_flange_thickness]);
    
  translate ([0,brace_height - brace_flange_thickness,brace_thickness]) cube([solar_panel_mount_width, brace_flange_thickness, brace_flange_length]);  
}

module brace_cuts () {
    
       //create rope hole
    translate ([solar_panel_mount_width/2, brace_height/2, -.9]) cylinder (99.9, rope_hole_diameter/2, rope_hole_diameter/2, center=false);
    
       //create ventilation slot
  translate([6,brace_height/2 - (brace_height - 5)/2 , -.9]) cube([20, brace_height - 6, 99.9]);
    
       //create ventilation slot
  translate([35,brace_height/2 - (brace_height - 5)/2 , -.9]) cube([20, brace_height - 6, 99.9]);
    
       //create ventilation slot
  translate([solar_panel_mount_width - 20 - 6,brace_height/2 - (brace_height - 5)/2 , -.9]) cube([20, brace_height - 6, 99.9]);
    
       //create ventilation slot
  translate([solar_panel_mount_width - 20 - 35,brace_height/2 - (brace_height - 5)/2 , -.9]) cube([20, brace_height - 6, 99.9]);
    
}
    

////////////////////////  SIDE TRIM ////////////////////////////

module side_trim_base () {
 cube([max_print_width, thin_trim_height + brace_height + solar_panel_thickness + thin_trim_thickness, trim_thickness]);
   
 //add wire flange
   translate ([max_print_width/2 - wire_flange_length/2, -wire_flange_height, 0]) cube([wire_flange_length, 15, trim_thickness]); 

}

 // vent slot values are tweaked to fit and symmetrical
 //so only 1 version can be printed 4 times 
 vent_slot_length = 33;  
 vent_slot_height = brace_height - thin_trim_thickness;
 vent_slot_start = 25;
 vent_slot_gap = 20;   

module side_trim_cuts () {
    
  //cut slot  
  translate ([-.9, thin_trim_height + brace_height, trim_thickness/2])  cube([999.9, solar_panel_thickness, 99.9]);
    
  //create ventilation slot 1 
  translate([vent_slot_start, thin_trim_height, -.9]) cube([vent_slot_length, vent_slot_height, 99.9]);
    
  //create ventilation slot 1 opposite end
  translate([max_print_width - vent_slot_start - vent_slot_length, thin_trim_height, -.9]) cube([vent_slot_length, vent_slot_height, 99.9]);
      
   
}

module side_trim_middle_slot() {
    

    
      //create ventilation slot 2
  translate([vent_slot_start + vent_slot_length + vent_slot_gap, thin_trim_height, -.9]) cube([vent_slot_length, vent_slot_height, 99.9]);
    
           //create ventilation slot 2 opposite end
  translate([max_print_width - vent_slot_start - vent_slot_length * 2 - vent_slot_gap, thin_trim_height, -.9]) cube([vent_slot_length, vent_slot_height, 99.9]);
  
       
}

module side_trim_box_cut_right(){
    
     // create inset for thin trim mounting of circuit box
  translate([solar_panel_length/2 - circuit_box_length, -99.9, thin_trim_thickness]) cube([999.9, thin_trim_height + 100, 99.9]);
    

         
}

module side_trim_box_cut_left() {
  
  // create inset for thin trim mounting of circuit box
translate([-.9, -99.9, thin_trim_thickness]) cube([max_print_width - (solar_panel_length/2 - circuit_box_length) + .9, thin_trim_height + 100, 99.9]);

}    

module side_trim_wire_slot_right () {
    translate ([max_print_width/2 - wire_flange_length/2 + thin_trim_thickness, -wire_flange_height + thin_trim_thickness , thin_trim_thickness]) cube ([99.9,thin_trim_height + wire_flange_height - thin_trim_thickness * 2, 99.9]);   
    
}

module side_trim_wire_slot_left () {
    translate ([-.9, -wire_flange_height + thin_trim_thickness , thin_trim_thickness]) cube ([max_print_width/2 + wire_flange_length/2 - thin_trim_thickness + .9,thin_trim_height + wire_flange_height - thin_trim_thickness * 2, 99.9]);   
    
}

module side_trim_right_mounting_holes() {
       //corner 
  translate ([batt_box_length/2 - batt_box_thickness, thin_trim_height + brace_height - 10,-.9])    cylinder (99.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1); 

       //inner 
  translate ([12.5, thin_trim_height + brace_height - 10,-.9])    cylinder (99.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1); 
    
    //brace
     translate ([batt_box_length/2 + 46, thin_trim_height + brace_height - 8,-.9])    cylinder (99.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1); 
    
    //aluminum bar
         translate ([batt_box_length/2 + 98, thin_trim_height + brace_height - 7.5,-.9])    cylinder (99.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1); 
             translate ([batt_box_length/2 + 98 + 15, thin_trim_height + brace_height - 7.5,-.9])    cylinder (99.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1); 

    
    
}

module side_trim_left_mounting_holes() {
           //corner 
  translate ([max_print_width - batt_box_length/2 + batt_box_thickness, thin_trim_height + brace_height - 10,-.9])    cylinder (99.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1); 

       //inner 
  translate ([max_print_width - 12.5, thin_trim_height + brace_height - 10,-.9])    cylinder (99.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1); 
    
        //brace
     translate ([max_print_width - batt_box_length/2 - 46, thin_trim_height + brace_height - 8,-.9])    cylinder (99.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1); 
    
    //aluminum bar
         translate ([max_print_width - batt_box_length/2 - 98, thin_trim_height + brace_height - 7.5,-.9])    cylinder (99.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1); 
             translate ([max_print_width - batt_box_length/2 - 98 - 15, thin_trim_height + brace_height - 7.5,-.9])    cylinder (99.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1); 
    
    
    }



//////////////////////// CORNER TRIM ////////////////////////

module corner_trim_base () {
 //end piece
 cube([trim_thickness, solar_panel_width + trim_thickness, trim_thickness]); 
 
 //corner side piece 1   
 cube([thin_trim_thickness + solar_panel_thickness + brace_height + thin_trim_height, trim_thickness, corner_trim_height]); 
 
 //corner side piece 2   
 translate ([0, solar_panel_width, 0])  cube([thin_trim_thickness + solar_panel_thickness + brace_height + thin_trim_height, trim_thickness, corner_trim_height]);
    
 //thick flange at end - hardcoded end mount width
 translate ([trim_thickness, (solar_panel_width + trim_thickness)/2 - 25/2, 0]) cube([brace_height - thin_trim_thickness + thin_trim_height, 25, brace_thickness]); 

}

module corner_trim_cuts () {
    
  //cut slot  
  translate ([trim_thickness/2 - solar_panel_thickness/2, trim_thickness/2, thin_trim_thickness]) cube([solar_panel_thickness, solar_panel_width, 999.9]);
  //create rope hole
    translate ([thin_trim_thickness + solar_panel_thickness + brace_height/2, (solar_panel_width + trim_thickness)/2, -.9]) cylinder (99.9, rope_hole_diameter/2, rope_hole_diameter/2, center=false);
    
  //create inset for thin trim mounting of circuit box
   translate([thin_trim_thickness + solar_panel_thickness + brace_height,thin_trim_thickness, thin_trim_thickness]) cube([999.9, circuit_box_width, 999.9]);
    
  //create ventilation slot
   translate([trim_thickness,-.9, corner_trim_height - 33]) cube([999.9, 999.9, 999.9]);
    
     //aluminum bar mounting holes   
             translate ([thin_trim_height + brace_height - 8, 300,53])    rotate ([90,0,0]) cylinder (999.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1); 
    
             translate ([thin_trim_height + brace_height - 8, 300,53- 15])    rotate ([90,0,0]) cylinder (999.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1); 
    
}

/////////////////////////////// WIRE SLOT /////////////////////

//vertical exit slot with rounded top for easy print
//used in batt frame and rotated for use in circuit boxes
module wire_slot() {
    cube ([wire_bundle_width, 99.9, wire_bundle_height - wire_bundle_width/2]);
    translate ([wire_bundle_width/2, 0, wire_bundle_height - wire_bundle_width/2]) rotate(a=[-90,0,0]) cylinder (99.9, wire_bundle_width/2, wire_bundle_width/2);
 
}

//////////////////////////////// BATT_FRAME //////////////////

module batt_box () {
 
difference() {    
batt_box_base();   
batt_box_cutouts();

}

difference() {
accelerometer_posts();
accelerometer_post_holes();
}


module batt_pulley () {

      //add base supports to pulley, angle is 12 degrees
       translate ([-4,-9,-3]) rotate ([0,-17.5,0]) cube ([4, 5, 10.5]);
   translate ([-4,4,-3]) rotate ([0,-17.5,0]) cube ([4, 5, 10.5]);

   difference() {rotate ([0,12,0]) pulley_bracket();
   //angle inside slot at 46 degrees for print
   translate ([19,-4,-1]) rotate ([0,-46,0]) cube ([9, 8, 99.9]);
   //angle outside
   translate ([-33,-10,0]) rotate ([0,44,0]) cube ([10, 20, 70]);
   }
   }
   
//add pulley brackets
translate ([-17.34,solar_panel_mount_width/2,8]) batt_pulley();
translate ([batt_box_length + 17.34,solar_panel_mount_width/2,8])  mirror ([1,0,0])batt_pulley();
   

//create module for top braces
module top_braces() {   
batt_brace_length = 94;
translate ([0,solar_panel_mount_width/2 - batt_brace_length/2,33.4 + 8]) cube ([8, batt_brace_length, 3]);
difference () {
translate ([0,solar_panel_mount_width/2 - batt_brace_length/2,33.4]) cube ([8, batt_brace_length, 8]);
translate ([0,solar_panel_mount_width/2 - batt_brace_length/2 - .1,33.4]) rotate ([0,45,0]) cube ([99, batt_brace_length + 99, 99]);
}


//long top brace 
difference () {
translate ([0,solar_panel_mount_width/2 - 70/2,batt_thickness]) cube ([8, 70, 25]);
translate ([2,solar_panel_mount_width/2 - batt_brace_length/2 - .1,batt_thickness]) rotate ([0,16,0]) cube ([99, 80 + 99, 99]);
}
}

//now add the top braces
top_braces();
translate ([batt_box_length, 0, 0]) mirror ([1,0,0]) top_braces();



module batt_box_base() {
 //create rectangular base
translate ([0, 0, 0]) cube([batt_box_length, solar_panel_mount_width, batt_box_height + motor_mount_depth]);   
    
//create angled edges
  translate ([0, 0, 0]) rotate(a=[0,0,180]) batt_box_angle();
  translate ([batt_box_length, solar_panel_mount_width, 0]) batt_box_angle();
    
//create side_brace support flanges

//allow .2mm space on all 4 sides
    
translate ([batt_box_length/2 + vent_slot_start + .2, -trim_thickness, thin_trim_thickness + .2]) batt_box_flange();
translate ([batt_box_length/2 - vent_slot_length - vent_slot_start - .2, -trim_thickness, thin_trim_thickness + .2]) batt_box_flange();    
translate ([batt_box_length/2 - vent_slot_start - .2, solar_panel_mount_width + trim_thickness, thin_trim_thickness]) rotate ([0,0,180]) batt_box_flange();
translate ([batt_box_length/2 + vent_slot_length + vent_slot_start + .2, solar_panel_mount_width + trim_thickness, thin_trim_thickness]) rotate ([0,0,180]) batt_box_flange();

//allow .2mm space on all 4 sides
module batt_box_flange() {    
 difference () {        
     cube ([vent_slot_length - .4, trim_thickness, vent_slot_height - .4]); 
     
      translate ([- .1, - 37.2, 30]) rotate ([-43, 0, 0]) cube ([vent_slot_length - .4 + .9, 99, vent_slot_height]);
     }    
        
}
}

module batt_box_angle() {
//extruded triangle (not 45 degrees)
rotate(a=[90,0,0]) linear_extrude(height = solar_panel_mount_width) {
 polygon( points=[[0,batt_box_height+motor_mount_depth],[batt_box_angle_width,0],[0,0]] );

}
}


module batt_box_cutouts() {    
    
//accel ramps
module accel_ramps() {
translate ([batt_box_length/2 - 16/2, 25.5, 6]) rotate ([16,0,0]) cube ([16, 34, 14]);
}
accel_ramps();
translate ([0, solar_panel_mount_width, 0]) mirror ([0,1,0]) accel_ramps();
    
    //cutout gaps to inset pulley brackets, 12 degree angle
    translate ([-17.34,solar_panel_mount_width/2,8]) rotate ([0,12,0]) pulley_bracket_slab();
    translate ([batt_box_length + 17.34,solar_panel_mount_width/2,8]) rotate ([0,-12,0]) mirror ([1,0,0]) pulley_bracket_slab();
    
    //rope hole cutout
    rope();

      //horizontal slots for pulley bolts
   translate ([-15.8, solar_panel_mount_width/2 - 8.9, 16.4]) rotate ([90,0,0]) cylinder (30, 5.25, 5.25);
   translate ([-15.8, solar_panel_mount_width/2 + 8.9 + 30, 16.4]) rotate ([90,0,0]) cylinder (30, 5.25, 5.25);
   translate ([batt_box_length + 15.8, solar_panel_mount_width/2 - 8.9, 16.4]) rotate ([90,0,0]) cylinder (30, 5.25, 5.25);
   translate ([batt_box_length + 15.8, solar_panel_mount_width/2 + 8.9 + 30, 16.4]) rotate ([90,0,0]) cylinder (30, 5.25, 5.25);

    
    //subtract battery box 1
    translate ([batt_box_length/2 - batt_width - accelerometer_width/2 - batt_box_thickness, solar_panel_mount_width/2 - batt_length/2, -.9]) cube([batt_width, batt_length, batt_thickness + 99.9]);
    
    //subtract battery box 2
    translate ([batt_box_length/2 + accelerometer_width/2 + batt_box_thickness, solar_panel_mount_width/2 - batt_length/2, -.9]) cube([batt_width, batt_length, batt_thickness + 99.9]);
    
    //lower interior to create accelerometer
    translate ([batt_box_length/2 - accelerometer_length/2, solar_panel_mount_width/2 - accelerometer_width/2,base_thickness ]) cube([accelerometer_length, accelerometer_width, 99.9]);  //subtract 4 mm to lower LIS3DH board from the top of the hole (where the rope would be) so MEMS chip is exactly on axis (-2 mm for board and chip, -2 mm for half of the rope width)
    
     //solar panel temp probe hole
    translate ([batt_box_length/2,solar_panel_mount_width/2,-.9]) cylinder(99.9, temp_probe_diameter/2,temp_probe_diameter/2 );
    
    //wire slot - long conduit path
    translate ([batt_box_thickness,solar_panel_mount_width - batt_box_thickness - cds_offset - 5, base_thickness + 3]) cube ([batt_box_length - batt_box_thickness * 2, 6, 99.9]);
    
            //wire slot - long conduit - bottom path
    translate ([batt_box_thickness,batt_box_thickness + cds_offset -1, base_thickness + 3]) cube ([batt_box_length - batt_box_thickness * 2, 6, 99.9]);
    
        //top aluminum battery cover slot
    translate ([batt_box_thickness,solar_panel_mount_width - batt_box_thickness - cds_offset + 3, base_thickness + 9]) cube ([batt_box_length - batt_box_thickness * 2, 1.4, 99.9]);
    
            //bottom aluminum battery cover slot
    translate ([batt_box_thickness,batt_box_thickness + cds_offset - 4.4, base_thickness + 9]) cube ([batt_box_length - batt_box_thickness * 2, 1.4, 99.9]);
    
    //remove extra plastic left over from wire slot and batt cover slots
        //12.7 is a fudge factor left over from trim_thickness error
    translate ([batt_box_thickness,solar_panel_mount_width - batt_box_thickness - cds_offset - 40, accelerometer_mount_height + 12.7 + 2.5]) cube ([batt_box_length - batt_box_thickness * 2, 43, 99.9]);
    
        
    //remove extra plastic left over from wire slot - bottom
    //12.7 is a fudge factor left over from trim_thickness error
    translate ([batt_box_thickness,batt_box_thickness + cds_offset - 3, accelerometer_mount_height + 12.7 + 2.5]) cube ([batt_box_length - batt_box_thickness * 2, 43, 99.9]);
    
     //solar panel power wires probe hole
    translate ([batt_box_length/2,solar_panel_mount_width - batt_box_thickness + temp_probe_diameter/2 - cds_offset - 6,-.9]) cylinder(99.9, temp_probe_diameter/2,temp_probe_diameter/2 );
    

    module battery_paths() {
    //wire slot corner - battery 1 paths
    translate ([batt_box_thickness,solar_panel_mount_width/2 + batt_length/2 - .9, base_thickness + 3]) cube ([10, 13, 99.9]);
    translate ([batt_box_thickness + batt_box_length/2 - 23.4,solar_panel_mount_width/2 + batt_length/2 - .9, base_thickness + 3]) cube ([10, 9, 99.9]);
    
      //wire slot corner - battery 1 - bottom path
    translate ([batt_box_thickness, solar_panel_mount_width/2 - batt_length/2 - 12.1, base_thickness + 3]) cube ([10, 12.2, 99.9]);
        translate ([batt_box_thickness + batt_box_length/2 - 23.4,solar_panel_mount_width/2 - batt_length/2 - 7, base_thickness + 3]) cube ([10, 7.1, 99.9]);
    }
    
    battery_paths();
    translate ([batt_box_length, 0, 0]) mirror ([1,0,0]) battery_paths();
      
      //wire slot corner - wire 1 path
  translate ([batt_box_thickness, solar_panel_mount_width/2 + 44,thin_trim_thickness + brace_height]) wire_slot();
  
      //wire slot corner - wire 1 - bottom path
    translate ([batt_box_thickness, - 81,thin_trim_thickness + brace_height]) wire_slot();
    
    
       //wire slot corner - wire 2 path
    translate ([batt_box_length - batt_box_thickness - wire_bundle_width,solar_panel_mount_width/2 + 44, thin_trim_thickness + brace_height]) wire_slot();
    
           //wire slot corner - wire 2 - bottom - path
    translate ([batt_box_length - batt_box_thickness - wire_bundle_width, - 81, thin_trim_thickness + brace_height]) wire_slot();
 
    
    //also add spot for battery box temp probe
       
    //gasket mount hole depression - pad with extra 5 mm of plastic to reinforce hex head
    
    //4 inside box
   translate ([27,10,base_thickness + 5 + 5 + 3.5]) cylinder(999.9, 6, 6);
    translate ([batt_box_length - 27,10,base_thickness + 5 + 5 + 3.5]) cylinder(999.9, 6, 6);
    translate ([27,solar_panel_mount_width - 10,base_thickness+ 5 + 5 + 3.5]) cylinder(999.9, 6, 6);
    translate ([batt_box_length - 27,solar_panel_mount_width - 10,base_thickness+ 5 + 5 + 3.5]) cylinder(999.9, 6, 6);
    
     //hex hole (pad with extra 5mm plastic to reinforce)
    translate ([27,10,base_thickness + 5]) cylinder(999.9, hex_diameter_8_32/2 + .2, hex_diameter_8_32/2 + .2, $fn=6);
    translate ([batt_box_length - 27,10,base_thickness + 5]) cylinder(999.9, hex_diameter_8_32/2 + .2, hex_diameter_8_32/2 + .2, $fn=6);
    translate ([27,solar_panel_mount_width - 10,base_thickness+ 5 ]) cylinder(999.9, hex_diameter_8_32/2 + .2, hex_diameter_8_32/2 + .2, $fn=6);
    translate ([batt_box_length - 27,solar_panel_mount_width - 10,base_thickness+ 5]) cylinder(999.9, hex_diameter_8_32/2 + .2, hex_diameter_8_32/2 + .2, $fn=6);
    
    //4 on outer angles
    translate ([-10,10,base_thickness + 5 + 5 + 3.5]) cylinder(999.9, 6, 6);
    translate ([batt_box_length + 10,10,base_thickness + 5 + 5 + 3.5]) cylinder(999.9, 6, 6);
    translate ([-10,solar_panel_mount_width - 10,base_thickness + 5 + 5 + 3.5]) cylinder(999.9, 6, 6);
    translate ([batt_box_length + 10,solar_panel_mount_width - 10,base_thickness + 5 + 5 + 3.5]) cylinder(999.9, 6, 6);
    
    //4 on inner angles
    translate ([-7,45,base_thickness + 15 + 5 + 3.5]) cylinder(999.9, 6, 6);
    translate ([batt_box_length + 7,45,base_thickness + 15 + 5 + 3.5]) cylinder(999.9, 6, 6);
    translate ([-7,solar_panel_mount_width - 45,base_thickness + 15 + 5 + 3.5]) cylinder(999.9, 6, 6);
    translate ([batt_box_length + 7,solar_panel_mount_width - 45,base_thickness + 15 + 5 + 3.5]) cylinder(999.9, 6, 6);
    
        //4 hex hole on outer angles
    translate ([-10,10,base_thickness + 5 + 3.5]) rotate ([0,0,90]) cylinder(999.9, hex_diameter_8_32/2 + .2, hex_diameter_8_32/2 + .2, $fn=6);
    translate ([batt_box_length + 10,10,base_thickness + 5 + 3.5]) rotate ([0,0,90]) cylinder(999.9, hex_diameter_8_32/2 + .2, hex_diameter_8_32/2 + .2, $fn=6);
    translate ([-10,solar_panel_mount_width - 10,base_thickness + 5 + 3.5]) rotate ([0,0,90]) cylinder(999.9, hex_diameter_8_32/2 + .2, hex_diameter_8_32/2 + .2, $fn=6);
    translate ([batt_box_length + 10,solar_panel_mount_width - 10,base_thickness + 5 + 3.5]) rotate ([0,0,90]) cylinder(999.9, hex_diameter_8_32/2 + .2, hex_diameter_8_32/2 + .2, $fn=6);
    
    //4 hex hole on inner angles
    translate ([-7,45,base_thickness + 15 + 3.5]) rotate ([0,0,90]) cylinder(999.9, hex_diameter_8_32/2 + .2, hex_diameter_8_32/2 + .2, $fn=6);
    translate ([batt_box_length + 7,45,base_thickness + 15 + 3.5]) rotate ([0,0,90]) cylinder(999.9, hex_diameter_8_32/2 + .2, hex_diameter_8_32/2 + .2, $fn=6);
    translate ([-7,solar_panel_mount_width - 45,base_thickness + 15 + 3.5]) rotate ([0,0,90]) cylinder(999.9, hex_diameter_8_32/2 + .2, hex_diameter_8_32/2 + .2, $fn=6);
    translate ([batt_box_length + 7,solar_panel_mount_width - 45,base_thickness + 15 + 3.5]) rotate ([0,0,90]) cylinder(999.9, hex_diameter_8_32/2 + .2, hex_diameter_8_32/2 + .2, $fn=6);
    
    //2 inside gyro cylinder
    translate ([batt_box_length/2,10,base_thickness+ 5 + 5 + 3.5]) cylinder(999.9, 6, 6);
    translate ([batt_box_length/2,solar_panel_mount_width - 10,base_thickness+ 5 + 5 + 3.5]) cylinder(999.9, 6, 6);
    
        
    //2 hex hole inside gyro cylinder
    translate ([batt_box_length/2,10,base_thickness + 5 + 3.5]) cylinder(999.9, hex_diameter_8_32/2 + .2, hex_diameter_8_32/2 + .2, $fn=6);
    translate ([batt_box_length/2,solar_panel_mount_width - 10,base_thickness+ 5 + 3.5]) cylinder(999.9, hex_diameter_8_32/2 + .2, hex_diameter_8_32/2 + .2, $fn=6);
    
        //gasket mount holes (diameter outer_diameter_8_32/2)
    //4 inside box (fixed for 8_32)
    translate ([27,10,-.9]) cylinder(999.9, outer_diameter_8_32/2 + .1, outer_diameter_8_32/2 + .1);
    translate ([batt_box_length - 27,10,-.9]) cylinder(999.9, outer_diameter_8_32/2 + .1,outer_diameter_8_32/2 + .1);
    translate ([27,solar_panel_mount_width - 10,-.9]) cylinder(999.9, outer_diameter_8_32/2 + .1, outer_diameter_8_32/2 + .1);
    translate ([batt_box_length - 27,solar_panel_mount_width - 10,-.9]) cylinder(999.9, outer_diameter_8_32/2 + .1, outer_diameter_8_32/2 + .1);
    
    //4 on outer angles
    translate ([-10,10,-.9]) cylinder(999.9, outer_diameter_8_32/2 + .1, outer_diameter_8_32/2 + .1);
    translate ([batt_box_length + 10,10,-.9]) cylinder(999.9, outer_diameter_8_32/2 + .1, outer_diameter_8_32/2 + .1);
    translate ([-10,solar_panel_mount_width - 10,-.9]) cylinder(999.9, outer_diameter_8_32/2 + .1, outer_diameter_8_32/2 + .1);
    translate ([batt_box_length + 10,solar_panel_mount_width - 10,-.9]) cylinder(999.9, outer_diameter_8_32/2 + .1, outer_diameter_8_32/2 + .1);
    
     //4 on inner angles
    translate ([-7,45,-.9]) cylinder(999.9, outer_diameter_8_32/2 + .1, outer_diameter_8_32/2 + .1);
    translate ([batt_box_length + 7,45,-.9]) cylinder(999.9, outer_diameter_8_32/2 + .1, outer_diameter_8_32/2 + .1);
    translate ([-7,solar_panel_mount_width - 45,-.9]) cylinder(999.9, outer_diameter_8_32/2 + .1, outer_diameter_8_32/2 + .1);
    translate ([batt_box_length + 7,solar_panel_mount_width - 45,-.9]) cylinder(999.9, outer_diameter_8_32/2 + .1, outer_diameter_8_32/2 + .1);
    
    //2 inside gyro cylinder
    translate ([batt_box_length/2,10,-.9]) cylinder(999.9, outer_diameter_8_32/2 + .1, outer_diameter_8_32/2 + .1);
    translate ([batt_box_length/2,solar_panel_mount_width - 10,-.9]) cylinder(999.9, outer_diameter_8_32/2 + .1, outer_diameter_8_32/2 + .1);
    
 //gondola screw holes (these are shifted off wrong, but had to make them wrong to match gondola holes - one side is too close to wire holes)
    //add .4mm buffer for holes so that gondola holes match at 8mm x 8 mm corners (good compromise from bad gondola print)
    module gondola_screw_holes() {
    //4 inside box
    //screw hole, all the way through
    translate ([batt_box_thickness + 8 + .4,batt_box_thickness + 8 + .4,-.1]) cylinder(999.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1);
    
    //triangle inset on bottom for 6mm round head/epoxy
    //at 7mm radius, it is about 1mm larger than round head
    translate ([batt_box_thickness + 8 + .4,batt_box_thickness + 8 + .4,-.1]) rotate ([0,0,180]) cylinder(3, 7, 7, $fn=3);
    //tapered triangle inset about it to allow printing ceiling
      translate ([batt_box_thickness + 8 + .4,batt_box_thickness + 8 + .4,2.89]) rotate ([0,0,180]) cylinder(7, 7, 0, $fn=3);
      
     //inset on top for hex nut
     translate ([batt_box_thickness + 8 + .4,batt_box_thickness + 8 + .4,batt_thickness+motor_mount_depth - 3]) cylinder(9.9, hex_diameter_6_32/2 + .2, hex_diameter_6_32/2+ .2, $fn=6);
    }
    
    gondola_screw_holes(); 
    translate ([0, solar_panel_mount_width, 0]) mirror ([0,-1,0])  gondola_screw_holes();  
     translate ([batt_box_length, solar_panel_mount_width, 0]) mirror ([-1,0,0]) mirror ([0,-1,0])  gondola_screw_holes();  
      translate ([batt_box_length, 0, 0]) mirror ([-1,0,0])  gondola_screw_holes(); 
    
  

 //lower interior to create inset around drip edge   
    translate ([batt_box_thickness, batt_box_thickness, batt_thickness+motor_mount_depth]) cube([batt_box_length - batt_box_thickness * 2, solar_panel_mount_width - batt_box_thickness * 2, 99.9]); 
    
     //lower interior to create inset for gyro mechanism and motor
     //it only provides 1 mm around the edges - very close.. try to change or print gyro disc smaller than an actual cd
     //12.7 is a fudge factor left over from trim_thickness error
      translate ([batt_box_length/2,solar_panel_mount_width/2,accelerometer_mount_height + 12.7 + 2.5]) cylinder(23,(gyro_diameter + 2)/2,(gyro_diameter + 2)/2); 
      
      //add another gyro cylinder at top to create 2mm bevel to add support
       translate ([batt_box_length/2,solar_panel_mount_width/2,accelerometer_mount_height + 12.7 + 2.5 + 22.9]) cylinder(3,(gyro_diameter + 2)/2,(gyro_diameter + 2)/2 - 2); 
      
      
     echo ("battery thickness is: ", batt_thickness);
     echo ("top of accelerometer is: ", accelerometer_mount_height + 2.5);
     echo ("gondola cover leaves accelerometer clearance of : ", 4.5 - 2.5);
    
    //rotated cylinder for side holes (fixed for 6_32)
    module side_holes() {
    translate ([0,-7,0]) rotate(a=[-90,0,0]) cylinder (7.1, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1); 
    translate ([0,-12,0]) rotate(a=[-90,90,0]) cylinder(5.1, hex_diameter_6_32/2 + .1, hex_diameter_6_32/2 + .1, $fn=6);
     //rotated square since horizontal big circles cannot be printed
     translate ([0,-20.5,0]) rotate(a=[-90,0,0]) cylinder (10.6, 7, 7, $fn = 4); 
}
    
    //side bolt holes
       //corner 1 
  translate ([batt_box_thickness, solar_panel_mount_width,10]) side_holes();

       //inner 1 
  translate ([batt_box_length/2 - 12.5, solar_panel_mount_width,10]) side_holes();

       //corner 2
  translate ([batt_box_length - batt_box_thickness, solar_panel_mount_width,10]) mirror ([0,0,-1]) side_holes();

       //inner 2
  translate ([batt_box_length - batt_box_length/2 + 12.5, solar_panel_mount_width,10]) mirror ([0,0,-1]) side_holes();

       //corner 3
  translate ([batt_box_thickness, 0,10]) mirror ([0,-1,0]) side_holes();

       //inner 3
  translate ([batt_box_length/2 - 12.5, 0,10]) mirror ([0,-1,0])side_holes();

       //corner 4
  translate ([batt_box_length - batt_box_thickness, 0,10]) mirror ([0,-1,0]) mirror ([0,0,-1]) side_holes();

       //inner 4
  translate ([batt_box_length - batt_box_length/2 + 12.5, 0,10]) mirror ([0,-1,0]) mirror ([0,0,-1]) side_holes();


  

    
}

module accelerometer_posts () {
    

            // accelerometer mounting posts    
translate ([batt_box_length/2 + accelerometer_length/2 - 2.75, solar_panel_mount_width/2 - accelerometer_width/2 + 2.4,0]) cylinder (accelerometer_mount_height, accelerometer_screw_mount_diameter/2 + 1, accelerometer_screw_mount_diameter/2);

translate ([batt_box_length/2 + accelerometer_length/2 - 2.75, solar_panel_mount_width/2 - accelerometer_width/2 + accelerometer_mount_width + 2.4,0]) cylinder (accelerometer_mount_height, accelerometer_screw_mount_diameter/2 + 1, accelerometer_screw_mount_diameter/2);
    
    //big middle accelerometer post
    // accelerometer mounting posts    
translate ([batt_box_length/2 - accelerometer_length/2 + accelerometer_width/2 - 5/2, solar_panel_mount_width/2 - accelerometer_width + accelerometer_length/2,0]) cube ([5, accelerometer_length, accelerometer_mount_height]);
    

    
}

module accelerometer_post_holes() {
    
        // accelerometer mounting holes    
translate ([batt_box_length/2 + accelerometer_length/2 - 2.75, solar_panel_mount_width/2 - accelerometer_width/2 + 2.4,base_thickness]) cylinder (99.9, accelerometer_screw_mount_inner_diameter/2, accelerometer_screw_mount_inner_diameter/2);

translate ([batt_box_length/2 + accelerometer_length/2 - 2.75, solar_panel_mount_width/2 - accelerometer_width/2 + accelerometer_mount_width + 2.4,base_thickness]) cylinder (99.9, accelerometer_screw_mount_inner_diameter/2, accelerometer_screw_mount_inner_diameter/2);
    
     //solar panel temp probe hole
    translate ([batt_box_length/2,solar_panel_mount_width/2,-.9]) cylinder(99.9, 4, 4);
    
}


}



//////////////////////////////// GONDOLA  /////////////////////
module longfin() {
//extruded triangle (not 45 degrees)
rotate(a=[90,0,0]) linear_extrude(height = fin_thickness) {
 polygon( points=[[0,screwdriver_length+endcap_drip_edge],[max_gondola_length/2 - screwdriver_orange_length/2 - cylinder_thickness,0],[0,0]] );
}
}

module shortfin() {
//extruded triangle (45 degrees)   
rotate(a=[90,0,0]) linear_extrude(height = fin_thickness) {
 polygon( points=[[0,max_gondola_width/2 - screwdriver_orange_length/2 - cylinder_thickness],[max_gondola_width/2 - screwdriver_orange_length/2 - cylinder_thickness,0],[0,0]] );
}
}

module base_and_fins() {
    
    //create rectangular base
      cube([max_gondola_length, max_gondola_width, base_thickness]); 
        
    //center cylinder  
    translate ([max_gondola_length/2,max_gondola_width/2,base_thickness]) cylinder(screwdriver_length+endcap_drip_edge,screwdriver_orange_length/2 + cylinder_thickness,screwdriver_orange_length/2 + cylinder_thickness);  
    
  //add long fins  (add fudge factor to keep a manifold - add later)
  translate ([max_gondola_length/2 + screwdriver_orange_length/2 + cylinder_thickness, max_gondola_width/2 + fin_thickness/2, base_thickness]) longfin();
    
   translate ([max_gondola_length/2 - screwdriver_orange_length/2 - cylinder_thickness, max_gondola_width/2 - fin_thickness/2, base_thickness]) rotate(a=[0,0,180]) longfin();
    
  //add short fins 
    
       translate ([max_gondola_length/2 + fin_thickness/2, max_gondola_width/2 - screwdriver_orange_length/2 - cylinder_thickness, base_thickness]) rotate(a=[0,0,-90]) shortfin();
    
   translate ([max_gondola_length/2 - fin_thickness/2, max_gondola_width/2 + screwdriver_orange_length/2 + cylinder_thickness, base_thickness]) rotate(a=[0,0,90]) shortfin();

    }

module cut_holes() {
    
      //center rectangle part of screwdriver handle
      translate ([max_gondola_length/2,max_gondola_width/2,base_thickness/2 + screwdriver_length/2 + 9.9])  rotate (a=[0,0,0]) cube([screwdriver_orange_width, screwdriver_orange_length, screwdriver_length+ 9.9], center = true);  
    
      //now cut out a larger cylinder at the top, above the rectangular orange part of the handle
      translate ([max_gondola_length/2,max_gondola_width/2,base_thickness + screwdriver_cylinder_start]) cylinder(screwdriver_length,screwdriver_orange_length/2,screwdriver_orange_length/2);  
    
      //now cut out inset drip cap edge for endcap (this is a mistake and shouldn't subtract endcap_drip_edge, since now the drip_edge is twice as long
      translate ([max_gondola_length/2,max_gondola_width/2,base_thickness + screwdriver_length - endcap_drip_edge]) cylinder(screwdriver_length,screwdriver_orange_length/2 + 7,screwdriver_orange_length/2 + 7);  
    
     endcap_screw_holes();
           
      //motor wires
      translate ([max_gondola_length/2,max_gondola_width/2 + motor_hub_diameter/2 + 3,-.9]) cylinder(base_thickness + 9.9,1,1);  
      
      //gyro motor hub
      translate ([max_gondola_length/2,max_gondola_width/2,-.9]) cylinder(motor_hub_depth,motor_hub_diameter/2 + .1,motor_hub_diameter/2 + 1.);  
    
      //CdS wires (manually tweak) - 50 is just an overshoot for the hole
      translate ([max_gondola_length/2,max_gondola_width - cds_offset,-.9]) cylinder(50,1,1);  
      translate ([max_gondola_length/2,cds_offset,-.9]) cylinder(50,1,1);  
    
      //Hall effect wires (manually tweak) - use a few less than 117 to close up hole
      //change--add these to end cap and let wires pass through motor housing
      //translate ([max_gondola_length/2 - screwdriver_orange_length/2 - 5 - 10,max_gondola_width/2,-.9]) cylinder(117,1,1);  
      //translate ([max_gondola_length/2 + screwdriver_orange_length/2 + 5 + 10,max_gondola_width/2,-.9]) cylinder(117,1,1);
      
    //screw holes - fixed 
    translate ([8 + .25,8+ .25,base_thickness - 15]) cylinder(999.9, outer_diameter_6_32/2 + .1,outer_diameter_6_32/2 + .1);
    
    translate ([batt_box_length - batt_box_thickness - outer_diameter_6_32/2 - .1 - 8 - .25,8 + .25,base_thickness - 15]) cylinder(999.9, outer_diameter_6_32/2 + .1,outer_diameter_6_32/2 + .1);
    
    translate ([8 + .25, solar_panel_mount_width - batt_box_thickness - outer_diameter_6_32/2 + .1 - 8 - .25,base_thickness - 15]) cylinder(999.9, outer_diameter_6_32/2 + .1,outer_diameter_6_32/2 + .1);
    
    translate ([batt_box_length - batt_box_thickness - outer_diameter_6_32/2 - .1 - 8 - .25,solar_panel_mount_width - batt_box_thickness - outer_diameter_6_32/2 - .1 - 8 - .25,base_thickness - 15]) cylinder(999.9, outer_diameter_6_32/2 + .1, outer_diameter_6_32/2 + .1);
        
    
}

module endcap_screw_holes() {
      //endcap screw holes
      translate ([max_gondola_length/2 - screwdriver_orange_length/2 + 1, max_gondola_width/2 - 15,base_thickness + screwdriver_length - endcap_drip_edge - 10]) cylinder(11,1,1);  

      translate ([max_gondola_length/2 + screwdriver_orange_length/2 - 1,max_gondola_width/2 + 15,base_thickness + screwdriver_length - endcap_drip_edge - 10]) cylinder(11,1,1);  
    
          translate ([max_gondola_length/2 - screwdriver_orange_length/2 + 1,max_gondola_width/2 + 15,base_thickness + screwdriver_length - endcap_drip_edge - 10]) cylinder(11,1,1);  

      translate ([max_gondola_length/2 + screwdriver_orange_length/2 - 1,max_gondola_width/2 - 15,base_thickness + screwdriver_length - endcap_drip_edge - 10]) cylinder(11,1,1);  
    
}

/////////////////////////////// RADIO BOX ////////////////////////////////////////////

module radio_box() {
    difference () {
    radio_box_base();
    radio_box_cutouts();  
    }
        
    radio_box_mounting_posts();  
    radio_box_side_posts();
    radio_mount_blocks(); //inserts to stabilize top of radio
    
}



module radio_box_base() {
    circuit_box();



}

module radio_box_side_posts(){
      translate ([circuit_box_thickness + 25, circuit_box_width - circuit_box_thickness - 8, base_thickness]) cube([10, 8, screw_height]);
    
        translate ([circuit_box_thickness + 60, circuit_box_width - circuit_box_thickness - 8, base_thickness]) cube([10, 8, screw_height]);
    
     translate ([circuit_box_thickness + 25, circuit_box_thickness, base_thickness]) cube([10, 8, screw_height]);
    
         translate ([circuit_box_thickness + 60, circuit_box_thickness, base_thickness]) cube([10, 8, screw_height]);
    
    
    }
    
 //radio is lifted up 7 mm to allow more room for audio interface underneath   
 module radio_mount_blocks() {
     
    translate ([circuit_box_length - circuit_box_thickness - 17, circuit_box_width/2 - radio_width/2, base_thickness]) cube([17, radio_width, radio_screw_mount_height - 7]);
     translate ([circuit_box_length - circuit_box_thickness - 30, circuit_box_width/2 - radio_width/2, base_thickness]) cube([13, radio_width, 7]);
 }

module radio_box_mounting_posts() {

   base_expansion = 1.5;
//Radio mounting posts
   radio_x = 58.5;
   radio_y = 51.5;
   difference () {
    //add post 1
    translate ([radio_x,radio_y,base_thickness -.1]) cylinder(radio_screw_mount_height,radio_screw_mount_diameter/2 + base_expansion,radio_screw_mount_diameter/2); 
        
      //cut hole 1
      translate ([radio_x,radio_y,-base_thickness]) cylinder(99.9,radio_screw_mount_inner_diameter/2,radio_screw_mount_inner_diameter/2); 
    }  
    difference () {   
    //add post 2
        translate ([radio_x, radio_screw_mount_width + radio_y,base_thickness -.1]) cylinder(radio_screw_mount_height,radio_screw_mount_diameter/2 + base_expansion,radio_screw_mount_diameter/2); 
        
      //cut hole 2
      translate ([radio_x, radio_screw_mount_width + radio_y,-base_thickness]) cylinder(99.9,radio_screw_mount_inner_diameter/2,radio_screw_mount_inner_diameter/2); 
    }     

// Pi mounting posts    
translate ([circuit_box_thickness + 24,circuit_box_thickness + 8,base_thickness]) mounting_posts (pi_screw_mount_inner_diameter, pi_screw_mount_diameter, base_expansion, pi_mount_length, pi_mount_width, pi_mount_height);
    
    // Circuit board mounting posts    
translate ([circuit_box_thickness + 6.5,circuit_box_width - circuit_board_length - circuit_box_thickness + 1,base_thickness]) mounting_posts (circuit_board_screw_mount_inner_diameter, circuit_board_screw_mount_diameter, base_expansion, circuit_board_mount_width, circuit_board_mount_length, circuit_board_mount_height);
    
        // ESP board mounting posts    
translate ([circuit_box_thickness + 57.25,circuit_box_width - esp_length - circuit_box_thickness + 1.5,base_thickness]) mounting_posts (esp_screw_mount_inner_diameter, esp_screw_mount_diameter,base_expansion, esp_mount_width, esp_mount_length, esp_mount_height);
    
        // Pi camera board mounting posts    
translate ([circuit_box_length - pi_camera_length - circuit_box_thickness + 7,circuit_box_thickness + 13.5,base_thickness]) mounting_posts (pi_camera_screw_mount_inner_diameter, pi_camera_screw_mount_diameter,base_expansion,pi_camera_mount_width, pi_camera_mount_length, pi_camera_mount_height);
}    



module radio_box_cutouts() {
//radio bottom void
    translate ([circuit_box_length - radio_length - circuit_box_thickness, circuit_box_width/2 - radio_width/2, base_thickness]) cube([radio_length, radio_width, 99.9]); 
    
    //radio wires
    translate ([circuit_box_length - radio_wire_width - circuit_box_thickness - 38, radio_width/2 + radio_wire_length/2 - 28.5, base_thickness]) cube([radio_wire_width, radio_wire_length, 99.9]); 
    
    ///radio bottom wires
    translate ([38, circuit_box_width/2 - radio_width/2 + esp_width/2 - 6, base_thickness + 5]) cube([esp_length, esp_width, esp_height + 99.9]);
    
//subtract circuit board
    translate ([circuit_box_thickness, circuit_box_width - circuit_board_length - circuit_box_thickness - 5, base_thickness]) cube([circuit_board_width, circuit_board_length, esp_height + 99.9]);
    
//subtract pi
    translate ([circuit_box_thickness + 10, circuit_box_thickness + 3, base_thickness]) cube([pi_length, pi_width, esp_height + 99.9]);
    
    //subtract pi wire space
    translate ([circuit_box_thickness + 25, circuit_box_thickness + pi_width + 1- .9, base_thickness]) cube([pi_wire_width, pi_wire_length, esp_height + 99.9]);
    
    //subtract esp
    translate ([circuit_box_thickness + 54, circuit_box_width - esp_length - circuit_box_thickness -2, base_thickness]) cube([esp_width, esp_length, esp_height + 99.9]);
    
    
     //subtract empty box - size of esp (can be used for ATTiny 1634 in future
    translate ([circuit_box_thickness + 84, circuit_box_width - esp_length - circuit_box_thickness - 1, base_thickness]) cube([esp_length, esp_width, esp_height + 99.9]);
    
    // subtract camera 
    translate ([circuit_box_length - pi_camera_length - circuit_box_thickness - 5, circuit_box_thickness + 11, base_thickness]) cube ([pi_camera_length, pi_camera_width, 99.9]);
    
    //subtract camera ribbon cable
    translate ([circuit_box_length - pi_camera_wire_length - circuit_box_thickness - 15, circuit_box_thickness + 16, base_thickness + 18]) cube ([pi_camera_wire_length, pi_camera_wire_width, 99.9]);
    
    
    // subtract light 
    translate ([circuit_box_length - pi_camera_length - circuit_box_thickness, circuit_box_width - light_width - circuit_box_thickness - 16,base_thickness + esp_height - light_height]) cube ([light_length, light_width, 99.9]);
    
        //subtract light and esp wires
    translate ([50, circuit_box_width - 39, base_thickness + 15]) cube ([90, 5, 99.9]);
    
        //subtract void for acrylic separator, up .1 mm for manifold 
     translate([circuit_box_length -  47, circuit_box_thickness,circuit_box_height - circuit_box_drip_edge - circuit_box_thickness * 2 + .1]) acrylic_separator_void(); 
     

 //subtract wire bundle hole (1 mm needed to break through wall
    translate ([circuit_box_thickness + wire_bundle_width + 7,circuit_box_width - wire_bundle_width - circuit_box_thickness - 1,thin_trim_thickness]) rotate (a=[0,0,90]) wire_slot();
    
     //subtract wire bundle hole - bottom (1 mm needed to break through wall
    translate ([circuit_box_thickness + wire_bundle_width + 15,circuit_box_thickness + 1,thin_trim_thickness]) rotate (a=[0,0,90]) wire_slot();


}
//////////////////////////////// SWITCHING BOX /////////////////////////////////////////

module switching_box() {
    difference () {
    circuit_box();
    switching_box_cutouts();    
    }
    
    
 translate ([-10,137 ,0]) rotate ([0,0,-90]) batt_board_mounts(18.6,39.5);
    
    
    
 translate ([circuit_box_length - 51,circuit_box_width - 21 ,0]) rotate ([0,0,-45]) solar_board_mounts(0,0);

translate ([20,circuit_box_thickness + 15,base_thickness]) mounting_posts (dc_boost_screw_mount_inner_diameter, dc_boost_screw_mount_diameter, 2, dc_boost_screw_mount_length, dc_boost_screw_mount_width, dc_boost_mount_height);

    
// Circuit board 2 mounting posts    
translate ([circuit_box_length - circuit_board2_length - circuit_box_thickness + 5.75, 13,base_thickness]) mounting_posts (circuit_board2_screw_mount_inner_diameter, circuit_board2_screw_mount_diameter, 2, circuit_board2_mount_length, circuit_board2_mount_width, circuit_board2_mount_height);
    
}    

   module solar_board_mounts(x_offset, y_offset) {
    
   difference() {
       solar_board_posts();
       solar_board_holes();
   }
 
   module solar_board_posts () {

   translate ([x_offset,y_offset,base_thickness -.9]) cylinder(solar_board_screw_mount_height,solar_board_screw_mount_diameter/2 + 2,solar_board_screw_mount_diameter/2);
    
       translate ([solar_board_screw_mount1_length + x_offset,y_offset,base_thickness -.9]) cylinder(solar_board_screw_mount_height,solar_board_screw_mount_diameter/2 + 2,solar_board_screw_mount_diameter/2); 
       
      translate ([x_offset + solar_board_screw_mount2_offset,y_offset + solar_board_screw_mount_width,base_thickness -.9])  cylinder (solar_board_screw_mount_height,solar_board_screw_mount_diameter/2 + 2,solar_board_screw_mount_diameter/2);
    
     translate ([solar_board_screw_mount1_length + x_offset - solar_board_screw_mount2_offset,y_offset + solar_board_screw_mount_width,base_thickness -.9]) cylinder(solar_board_screw_mount_height,solar_board_screw_mount_diameter/2 + 2,solar_board_screw_mount_diameter/2); 
   }
   
      module solar_board_holes () {

   translate ([x_offset,y_offset,base_thickness]) cylinder(99.9,solar_board_screw_mount_inner_diameter/2,solar_board_screw_mount_inner_diameter/2);
    
       translate ([solar_board_screw_mount1_length + x_offset,y_offset,base_thickness]) cylinder(99.9,solar_board_screw_mount_inner_diameter/2,solar_board_screw_mount_inner_diameter/2); 
       
      translate ([x_offset + solar_board_screw_mount2_offset,y_offset + solar_board_screw_mount_width,base_thickness])  cylinder (99.9,solar_board_screw_mount_inner_diameter/2,solar_board_screw_mount_inner_diameter/2);
    
     translate ([solar_board_screw_mount1_length + x_offset - solar_board_screw_mount2_offset,y_offset + solar_board_screw_mount_width,base_thickness]) cylinder(99.9,solar_board_screw_mount_inner_diameter/2,solar_board_screw_mount_inner_diameter/2); 
   }
   
   
   }

module batt_board_mounts(x_offset, y_offset) {
    
   difference() {
       batt_board_posts(x_offset, y_offset);
       batt_board_holes(x_offset, y_offset);
   }
   
   }
    
   module batt_board_posts(x_offset,y_offset) { 
   //screw posts 
   translate ([x_offset,y_offset,base_thickness -.9]) cylinder(batt_board_screw_mount_height,batt_board_screw_mount_diameter/2 + 2,batt_board_screw_mount_diameter/2);
    
       translate ([batt_board_screw_mount_length + x_offset,y_offset,base_thickness -.9]) cylinder(batt_board_screw_mount_height,batt_board_screw_mount_diameter/2 + 2,batt_board_screw_mount_diameter/2); 
    
    //post posts
      translate ([x_offset,y_offset - batt_board_mount_width,base_thickness -.9]) cylinder(batt_board_screw_mount_height,batt_board_screw_mount_diameter/2 + 2,batt_board_screw_mount_diameter/2); 
    
      translate ([batt_board_screw_mount_length + x_offset,y_offset - batt_board_mount_width,base_thickness -.9]) cylinder(batt_board_screw_mount_height,batt_board_screw_mount_diameter/2 + 2,batt_board_screw_mount_diameter/2);
       
     //tiny posts
      translate ([x_offset - .35,y_offset - batt_board_mount_width,base_thickness -.9]) cylinder(batt_board_screw_mount_height + 2,batt_board_post_diameter/2,batt_board_post_diameter/2); 
    
      translate ([batt_board_mount_length + x_offset -.35,y_offset - batt_board_mount_width,base_thickness -.9]) cylinder(batt_board_screw_mount_height + 2,batt_board_post_diameter/2,batt_board_post_diameter/2);
        
   }
       

   module batt_board_holes(x_offset, y_offset) {
   
       //screw post holes
       translate ([x_offset,y_offset,base_thickness -.9]) cylinder(99.9,batt_board_screw_mount_inner_diameter/2,batt_board_screw_mount_inner_diameter/2);
    
       translate ([batt_board_screw_mount_length + x_offset,y_offset,base_thickness -.9]) cylinder(99.9,batt_board_screw_mount_inner_diameter/2,batt_board_screw_mount_inner_diameter/2); 
}


module switching_box_cutouts() {    
//subtract solar board   
    translate ([107, 126, base_thickness]) rotate (a=[0,0,-45]) linear_extrude(height = 99.9) {polygon( points=[[0,0],[6,0],[6,-50],[49,-50],[49,0],[52 + 2,0],[52 + 2,23 + 1], [26 + 1,43 + 2], [0,23 + 1]]);}

    
        
    //subtract batt board bottom layer
    translate ([circuit_box_thickness, circuit_box_width - circuit_box_thickness - batt_board_width - 10 - 21, base_thickness]) cube([batt_board_width + 2, batt_board_length + 2, esp_height + 99.9]);
    
       //subtract batt board top layer
    translate ([circuit_box_thickness, circuit_box_width - circuit_box_thickness - batt_board_width - 10 - 21 - 7.5, base_thickness + batt_board_screw_mount_height + 12]) cube([batt_board_width + 2, batt_board_length + 2 + 15 ,99.9]);
    
        //subtract dc boost
     translate ([circuit_box_thickness + 10, circuit_box_thickness + 10, base_thickness]) cube([dc_boost_length, dc_boost_width,  99.9]);
    
     //subtract circuit board 2
    translate ([circuit_box_length - circuit_board2_length - circuit_box_thickness, circuit_box_thickness + 5, base_thickness]) cube([circuit_board2_length, circuit_board2_width, 99.9]);
    
     //subtract wire bundle hole (1 mm needed to break through wall
    translate ([99.9 + circuit_box_length - (circuit_box_thickness + wire_bundle_width + 7),circuit_box_width - wire_bundle_width - circuit_box_thickness - 1,thin_trim_thickness]) rotate (a=[0,0,90]) wire_slot();
    
     //subtract wire bundle hole - bottom (1 mm needed to break through wall
    translate ([99.9 + circuit_box_length - (circuit_box_thickness + wire_bundle_width + 15),circuit_box_thickness + 1,thin_trim_thickness]) rotate (a=[0,0,90]) wire_slot();
    
        //subtract batt board wires
    translate ([circuit_box_thickness + batt_board_width, circuit_box_width - circuit_box_thickness - batt_board_width - 10 - 12, base_thickness]) cube([batt_board_wire_width, batt_board_wire_length, 99.9]);
    
    //subtract wire channel between circuit board and boost
      translate ([circuit_box_thickness + 10 + dc_boost_length - .9, circuit_box_thickness + 10 + dc_boost_width/2 - 2.5, base_thickness + dc_boost_mount_height]) cube([15, 5,  99.9]);
      
   //holes for batt board top layer
   
 
   
      translate ([circuit_box_thickness + 5,131,base_thickness + batt_board_screw_mount_height + 12 - 10]) cylinder(99.9,1.2,1.2); 
      
      translate ([circuit_box_thickness + 5,131 - 68.5,base_thickness + batt_board_screw_mount_height + 12 - 10]) cylinder(99.9,1.2,1.2); 
      
        translate ([circuit_box_thickness + 42,131,base_thickness + batt_board_screw_mount_height + 12 - 10]) cylinder(99.9,1.2,1.2); 
      
      translate ([circuit_box_thickness + 42,131 - 68.5,base_thickness + batt_board_screw_mount_height + 12 - 10]) cylinder(99.9,1.2,1.2); 
      
  
   
   
}

/////////////////////////////// CPU HOLDER ///////////////////////////
module cpu_holder () {

    difference() {
          batt_board_base();
 
      //screw holes
      translate ([3,4,-.9]) cylinder(99.9,1.5,1.5);         
      translate ([3 + 68.5, 4,-.9]) cylinder(99.9,1.5,1.5);
      translate ([3,41,-.90]) cylinder(99.9,1.5,1.5);
      translate ([3 + 68.5,41,-.9]) cylinder(99.9,1.5,1.5);  
        
      //zip tie holes
      translate ([16.5,12,-.9]) cube([2,4,99.9]);      
      translate ([68.5 - 13, 12,-.9]) cube([2,4,99.9]);   
      translate ([16.5,27,-.9]) cube([2,4,99.9]);      
      translate ([68.5 - 13, 27,-.9]) cube([2,4,99.9]);  
        
        
        
        
      //power switch access notch
      translate ([57,batt_board_width + 2 - 2 - 10,-.9]) cube([10,10.9,99.9]);  
        
      //inset for cpu
      translate([(cpu_holder_inset_width + 16)/2, 4, 2]) 
        cube([cpu_holder_inset_width, 99.9, 99.9]);  
          
    
}

}


module batt_board_base () {

cube([batt_board_length + 2 + 15 - 2, batt_board_width + 2 - 2,cpu_holder_thickness]);  
    
        translate([(cpu_holder_inset_width + 8)/2, 0, cpu_holder_thickness]) cube([cpu_holder_inset_width + 8, batt_board_width + 2 - 2, cpu_holder_inset_depth]);
    
    
    
}

/////////////////////////////// CIRCUIT BOX ////////////////////////////////////
//This is the base model for the 2 side boxes


    module circuit_box () {
        
     module circuit_box_base() {
     //create rectangular base
translate ([0, 0, 0]) cube([circuit_box_length, circuit_box_width, circuit_box_height]); 
     }
         
 
difference() {    
  circuit_box_base();
    
 //lower interior to create inset around drip edge   
 translate ([circuit_box_thickness, circuit_box_thickness, circuit_box_height - circuit_box_thickness - circuit_box_drip_edge]) cube([circuit_box_length - circuit_box_thickness * 2, circuit_box_width - circuit_box_thickness * 2, 99.9]); 
 translate ([0,0,base_thickness]) bottom_holes();   
    
}
}

////////////////// MOUNTING POSTS ///////////////////////////

module mounting_posts (inner_diameter, outer_diameter, base_expansion, x_len, y_len, height) {
     
    difference () {
    //add post 1
    translate ([0,0,0]) cylinder(height,outer_diameter/2 + base_expansion,outer_diameter/2); 
        
      //cut hole 1
      translate ([0,0,base_thickness]) cylinder(99.9,inner_diameter/2,inner_diameter/2); 
    }  
    difference () {   
    //add post 2
        translate ([x_len,0,0]) cylinder(height,outer_diameter/2 + base_expansion,outer_diameter/2); 
        
      //cut hole 2
      translate ([x_len,0,base_thickness]) cylinder(99.9,inner_diameter/2,inner_diameter/2); 
    }     
    
        difference () {   
    //add post 3
        translate ([x_len,y_len,0]) cylinder(height,outer_diameter/2 + base_expansion,outer_diameter/2); 
        
      //cut hole 3
      translate ([x_len,y_len,base_thickness]) cylinder(99.9,inner_diameter/2,inner_diameter/2); 
    }     
        difference () {   
    //add post 4
        translate ([0,y_len,0]) cylinder(height,outer_diameter/2 + base_expansion,outer_diameter/2); 
        
      //cut hole 4
      translate ([0,y_len,base_thickness]) cylinder(99.9,inner_diameter/2,inner_diameter/2); 
    }     
    
}

//////////////  BOTTOM PANEL HOLES //////////////////
module bottom_holes () {
translate ([4,4,wire_bundle_height + 2]) cylinder(99.9,1,1); 
translate ([circuit_box_length - 4,4,wire_bundle_height + 2]) cylinder(99.9,1,1); 
translate ([circuit_box_length - 4,circuit_box_width - 4,wire_bundle_height + 2]) cylinder(99.9,1,1); 
translate ([4,circuit_box_width - 4,wire_bundle_height + 2]) cylinder(99.9,1,1); 
}
    
